# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import Scanners, ScannerType


class ScannersTestCase(unittest.TestCase):
    def test_create_scanner(self):
        request = Scanners.create_scanner(
            "foo",
            "localhost",
            1234,
            scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
            credential_id="c1",
        )

        self.assertEqual(
            bytes(request),
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>localhost</host>"
            b"<port>1234</port>"
            b"<type>2</type>"
            b'<credential id="c1"/>'
            b"</create_scanner>",
        )

    def test_create_scanner_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                None,
                "localhost",
                1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "",
                "localhost",
                1234,
                scanner_type="2",
                credential_id="c1",
            )

    def test_create_scanner_missing_host(self):
        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                None,
                1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                "",
                1234,
                scanner_type="2",
                credential_id="c1",
            )

    def test_create_scanner_missing_port(self):
        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                "localhost",
                None,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                "localhost",
                "",
                scanner_type="2",
                credential_id="c1",
            )

    def test_create_scanner_missing_scanner_type(self):
        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                "localhost",
                1234,
                scanner_type=None,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                "localhost",
                1234,
                scanner_type="",
                credential_id="c1",
            )

    def test_create_scanner_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                "localhost",
                1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id=None,
            )

        with self.assertRaises(RequiredArgument):
            Scanners.create_scanner(
                "foo",
                "localhost",
                1234,
                scanner_type="2",
                credential_id="",
            )

    def test_create_scanner_invalid_scanner_type(self):
        with self.assertRaises(InvalidArgument):
            Scanners.create_scanner(
                "foo",
                "localhost",
                1234,
                scanner_type="invalid",
                credential_id="c1",
            )

    def test_create_scanner_with_ca_pub(self):
        request = Scanners.create_scanner(
            "foo",
            "localhost",
            1234,
            scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
            credential_id="c1",
            ca_pub="foo",
        )

        self.assertEqual(
            bytes(request),
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>localhost</host>"
            b"<port>1234</port>"
            b"<type>2</type>"
            b'<credential id="c1"/>'
            b"<ca_pub>foo</ca_pub>"
            b"</create_scanner>",
        )

    def test_create_scanner_with_comment(self):
        request = Scanners.create_scanner(
            "foo",
            "localhost",
            1234,
            scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
            credential_id="c1",
            comment="foo",
        )

        self.assertEqual(
            bytes(request),
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>localhost</host>"
            b"<port>1234</port>"
            b"<type>2</type>"
            b'<credential id="c1"/>'
            b"<comment>foo</comment>"
            b"</create_scanner>",
        )

    def test_modify_scanner(self):
        request = Scanners.modify_scanner("s1")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1"/>',
        )

    def test_modify_scanner_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            Scanners.modify_scanner(None)

        with self.assertRaises(RequiredArgument):
            Scanners.modify_scanner("")

    def test_modify_scanner_with_comment(self):
        request = Scanners.modify_scanner("s1", comment="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1">'
            b"<comment>foo</comment>"
            b"</modify_scanner>",
        )

    def test_modify_scanner_with_host(self):
        request = Scanners.modify_scanner("s1", host="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1">'
            b"<host>foo</host>"
            b"</modify_scanner>",
        )

    def test_modify_scanner_with_port(self):
        request = Scanners.modify_scanner("s1", port=1234)

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1">'
            b"<port>1234</port>"
            b"</modify_scanner>",
        )

        request = Scanners.modify_scanner("s1", port="1234")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1">'
            b"<port>1234</port>"
            b"</modify_scanner>",
        )

    def test_modify_scanner_with_name(self):
        request = Scanners.modify_scanner("s1", name="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1">'
            b"<name>foo</name>"
            b"</modify_scanner>",
        )

    def test_modify_scanner_with_ca_pub(self):
        request = Scanners.modify_scanner("s1", ca_pub="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1">'
            b"<ca_pub>foo</ca_pub>"
            b"</modify_scanner>",
        )

    def test_modify_scanner_with_credential_id(self):
        request = Scanners.modify_scanner("s1", credential_id="c1")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1">'
            b'<credential id="c1"/>'
            b"</modify_scanner>",
        )

    def test_modify_scanner_with_scanner_type(self):
        request = Scanners.modify_scanner(
            "s1", scanner_type=ScannerType.OPENVAS_SCANNER_TYPE
        )

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1"><type>2</type></modify_scanner>',
        )

        request = Scanners.modify_scanner("s1", scanner_type="2")

        self.assertEqual(
            bytes(request),
            b'<modify_scanner scanner_id="s1"><type>2</type></modify_scanner>',
        )

    def test_modify_scanner_invalid_scanner_type(self):
        with self.assertRaises(InvalidArgument):
            Scanners.modify_scanner("s1", scanner_type="invalid")

        with self.assertRaises(ValueError):
            Scanners.modify_scanner("s1", scanner_type="")

        with self.assertRaises(InvalidArgument):
            Scanners.modify_scanner("s1", scanner_type="-1")

        with self.assertRaises(InvalidArgument):
            Scanners.modify_scanner("s1", scanner_type=1)

    def test_get_scanners(self):
        request = Scanners.get_scanners()

        self.assertEqual(bytes(request), b"<get_scanners/>")

    def test_get_scanners_with_filter_string(self):
        request = Scanners.get_scanners(filter_string="foo=bar")

        self.assertEqual(
            bytes(request),
            b'<get_scanners filter="foo=bar"/>',
        )

    def test_get_scanners_with_filter_id(self):
        request = Scanners.get_scanners(filter_id="f1")

        self.assertEqual(
            bytes(request),
            b'<get_scanners filt_id="f1"/>',
        )

    def test_get_scanners_with_trash(self):
        request = Scanners.get_scanners(trash=True)

        self.assertEqual(
            bytes(request),
            b'<get_scanners trash="1"/>',
        )

        request = Scanners.get_scanners(trash=False)

        self.assertEqual(
            bytes(request),
            b'<get_scanners trash="0"/>',
        )

    def test_get_scanners_with_details(self):
        request = Scanners.get_scanners(details=True)

        self.assertEqual(
            bytes(request),
            b'<get_scanners details="1"/>',
        )

        request = Scanners.get_scanners(details=False)

        self.assertEqual(
            bytes(request),
            b'<get_scanners details="0"/>',
        )

    def test_get_scanner(self):
        request = Scanners.get_scanner("s1")

        self.assertEqual(
            bytes(request),
            b'<get_scanners scanner_id="s1" details="1"/>',
        )

    def test_get_scanner_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            Scanners.get_scanner(None)

        with self.assertRaises(RequiredArgument):
            Scanners.get_scanner("")

    def test_verify_scanner(self):
        request = Scanners.verify_scanner("s1")

        self.assertEqual(
            bytes(request),
            b'<verify_scanner scanner_id="s1"/>',
        )

    def test_verify_scanner_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            Scanners.verify_scanner(None)

        with self.assertRaises(RequiredArgument):
            Scanners.verify_scanner("")

    def test_clone_scanner(self):
        request = Scanners.clone_scanner("s1")

        self.assertEqual(
            bytes(request),
            b"<create_scanner><copy>s1</copy></create_scanner>",
        )

    def test_clone_scanner_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            Scanners.clone_scanner(None)

        with self.assertRaises(RequiredArgument):
            Scanners.clone_scanner("")

    def test_delete_scanner(self):
        request = Scanners.delete_scanner("s1")

        self.assertEqual(
            bytes(request),
            b'<delete_scanner scanner_id="s1" ultimate="0"/>',
        )

    def test_delete_scanner_ultimate(self):
        request = Scanners.delete_scanner("s1", ultimate=True)

        self.assertEqual(
            bytes(request),
            b'<delete_scanner scanner_id="s1" ultimate="1"/>',
        )

        request = Scanners.delete_scanner("s1", ultimate=False)

        self.assertEqual(
            bytes(request),
            b'<delete_scanner scanner_id="s1" ultimate="0"/>',
        )

    def test_delete_scanner_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            Scanners.delete_scanner(None)

        with self.assertRaises(RequiredArgument):
            Scanners.delete_scanner("")



# SPDX-FileCopyrightText: 2021-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_task import GmpCloneTaskTestMixin
from .test_create_agent_group_task import GmpCreateAgentGroupTaskTestMixin
from .test_create_container_image_task import (
    GmpCreateContainerImageTaskTestMixin,
)
from .test_create_container_task import GmpCreateContainerTaskTestMixin
from .test_create_import_task import GmpCreateImportTaskTestMixin
from .test_create_task import GmpCreateTaskTestMixin
from .test_delete_task import GmpDeleteTaskTestMixin
from .test_get_task import GmpGetTaskTestMixin
from .test_get_tasks import GmpGetTasksTestMixin
from .test_modify_task import GmpModifyTaskTestMixin
from .test_move_task import GmpMoveTaskTestMixin
from .test_resume_task import GmpResumeTaskTestMixin
from .test_start_task import GmpStartTaskTestMixin
from .test_stop_task import GmpStopTaskTestMixin

__all__ = (
    "GmpCloneTaskTestMixin",
    "GmpCreateAgentGroupTaskTestMixin",
    "GmpCreateContainerImageTaskTestMixin",
    "GmpCreateContainerTaskTestMixin",
    "GmpCreateImportTaskTestMixin",
    "GmpCreateTaskTestMixin",
    "GmpDeleteTaskTestMixin",
    "GmpGetTaskTestMixin",
    "GmpGetTasksTestMixin",
    "GmpModifyTaskTestMixin",
    "GmpMoveTaskTestMixin",
    "GmpResumeTaskTestMixin",
    "GmpStartTaskTestMixin",
    "GmpStopTaskTestMixin",
)



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.protocols.gmp.requests._entity_id import EntityID
from gvm.utils import to_bool, to_comma_list
from gvm.xml import XmlCommand


class OCIImageTargets:
    @classmethod
    def create_oci_image_target(
        cls,
        name: str,
        image_references: list[str],
        *,
        comment: str | None = None,
        credential_id: EntityID | None = None,
    ) -> Request:
        """Create a new OCI image target

        Args:
            name: Name of the target
            image_references: List of OCI image URLs to scan
            comment: Comment for the target
            credential_id: UUID of a credential to use on target
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_oci_image_target.__name__, argument="name"
            )

        if not image_references:
            raise RequiredArgument(
                function=cls.create_oci_image_target.__name__,
                argument="image_references",
            )

        cmd = XmlCommand("create_oci_image_target")
        cmd.add_element("name", name)
        cmd.add_element("image_references", to_comma_list(image_references))

        if comment:
            cmd.add_element("comment", comment)

        if credential_id:
            cmd.add_element("credential", attrs={"id": str(credential_id)})

        return cmd

    @classmethod
    def modify_oci_image_target(
        cls,
        oci_image_target_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        image_references: list[str] | None = None,
        credential_id: EntityID | None = None,
    ) -> Request:
        """Modify an existing target.

        Args:
            oci_image_target_id: UUID of target to modify.
            comment: Comment on target.
            name: Name of target.
            image_references: List of OCI image URLs.
            credential_id: UUID of credential to use on target.
        """
        if not oci_image_target_id:
            raise RequiredArgument(
                function=cls.modify_oci_image_target.__name__,
                argument="oci_image_target_id",
            )

        cmd = XmlCommand("modify_oci_image_target")
        cmd.set_attribute("oci_image_target_id", str(oci_image_target_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if image_references:
            cmd.add_element("image_references", to_comma_list(image_references))

        if credential_id:
            cmd.add_element("credential", attrs={"id": str(credential_id)})

        return cmd

    @classmethod
    def clone_oci_image_target(cls, oci_image_target_id: EntityID) -> Request:
        """Clone an existing OCI image target.

        Args:
            oci_image_target_id: UUID of an existing target to clone.
        """
        if not oci_image_target_id:
            raise RequiredArgument(
                function=cls.clone_oci_image_target.__name__,
                argument="oci_image_target_id",
            )

        cmd = XmlCommand("create_oci_image_target")
        cmd.add_element("copy", str(oci_image_target_id))
        return cmd

    @classmethod
    def delete_oci_image_target(
        cls, oci_image_target_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Delete an existing OCI image target.

        Args:
            oci_image_target_id: UUID of an existing target to delete.
            ultimate: Whether to remove entirely or to the trashcan.
        """
        if not oci_image_target_id:
            raise RequiredArgument(
                function=cls.delete_oci_image_target.__name__,
                argument="oci_image_target_id",
            )

        cmd = XmlCommand("delete_oci_image_target")
        cmd.set_attribute("oci_image_target_id", str(oci_image_target_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))
        return cmd

    @classmethod
    def get_oci_image_target(
        cls, oci_image_target_id: EntityID, *, tasks: bool | None = None
    ) -> Request:
        """Request a single OCI Image target.

        Args:
            oci_image_target_id: UUID of the target to request.
            tasks: Whether to include list of tasks that use the target
        """
        if not oci_image_target_id:
            raise RequiredArgument(
                function=cls.get_oci_image_target.__name__,
                argument="oci_image_target_id",
            )

        cmd = XmlCommand("get_oci_image_targets")
        cmd.set_attribute("oci_image_target_id", str(oci_image_target_id))

        if tasks is not None:
            cmd.set_attribute("tasks", to_bool(tasks))

        return cmd

    @classmethod
    def get_oci_image_targets(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        tasks: bool | None = None,
    ) -> Request:
        """Request a list of OCI image targets.

        Args:
            filter_string: Filter term to use for the query.
            filter_id: UUID of an existing filter to use for the query.
            trash: Whether to include targets in the trashcan.
            tasks: Whether to include list of tasks that use the target.
        """
        cmd = XmlCommand("get_oci_image_targets")
        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if tasks is not None:
            cmd.set_attribute("tasks", to_bool(tasks))

        return cmd



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
from tests.protocols.gmpv224.entities.scanners import GmpVerifyScannerTestMixin
from tests.protocols.gmpv227 import GMPTestCase
from tests.protocols.gmpv227.entities.scanners import (
    GmpCloneScannerTestMixin,
    GmpCreateScannerTestMixin,
    GmpDeleteScannerTestMixin,
    GmpGetScannersTestMixin,
    GmpGetScannerTestMixin,
    GmpModifyScannerTestMixin,
)


class GMPDeleteScannerTestCase(GmpDeleteScannerTestMixin, GMPTestCase):
    pass


class GMPGetScannerTestCase(GmpGetScannerTestMixin, GMPTestCase):
    pass


class GMPGetScannersTestCase(GmpGetScannersTestMixin, GMPTestCase):
    pass


class GMPCloneScannerTestCase(GmpCloneScannerTestMixin, GMPTestCase):
    pass


class GMPCreateScannerTestCase(GmpCreateScannerTestMixin, GMPTestCase):
    pass


class GMPModifyScannerTestCase(GmpModifyScannerTestMixin, GMPTestCase):
    pass


class GMPVerifyScannerTestCase(GmpVerifyScannerTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

from gvm.protocols.core import InvalidStateError, Response
from gvm.protocols.core._connection import (
    AwaitingResponseState,
    Context,
    ErrorState,
    InitialState,
    ReceivingDataState,
)


class RequestMock:
    def __init__(self, data: bytes) -> None:
        self._data = data

    def __bytes__(self) -> bytes:
        return self._data


class InitialStateTestCase(unittest.TestCase):
    def test_receive_data(self) -> None:
        context = MagicMock(spec=Context)
        state = InitialState()
        state.__set_context__(context)

        with self.assertRaisesRegex(InvalidStateError, "Invalid State"):
            state.receive_data(b"some data")

    def test_close(self) -> None:
        context = MagicMock(spec=Context)
        state = InitialState()
        state.__set_context__(context)

        state.close()

        context.__set_state__.assert_not_called()

    def test_send(self) -> None:
        context = MagicMock(spec=Context)
        state = InitialState()
        state.__set_context__(context)
        request = RequestMock(b"some data")

        data = state.send(request)
        self.assertEqual(data, b"some data")

        self.assertIsInstance(
            context.__set_state__.call_args[0][0], AwaitingResponseState
        )


class AwaitingResponseStateTestCase(unittest.TestCase):
    def test_receive_data(self) -> None:
        context = MagicMock(spec=Context)
        request = RequestMock(b"<start>")
        state = AwaitingResponseState(request)
        state.__set_context__(context)

        response = state.receive_data(b"<element>")
        self.assertIsNone(response)

        self.assertIsInstance(
            context.__set_state__.call_args[0][0], ReceivingDataState
        )

    def test_close(self) -> None:
        context = MagicMock(spec=Context)
        request = RequestMock(b"<start>")
        state = AwaitingResponseState(request)
        state.__set_context__(context)

        state.close()

        self.assertIsInstance(
            context.__set_state__.call_args[0][0], InitialState
        )

    def test_send(self) -> None:
        context = MagicMock(spec=Context)
        request = RequestMock(b"<start>")
        state = AwaitingResponseState(request)
        state.__set_context__(context)

        with self.assertRaisesRegex(InvalidStateError, "Invalid State"):
            another_request = RequestMock(b"<element>")
            state.send(another_request)


class ErrorStateTestCase(unittest.TestCase):
    def test_receive_data(self) -> None:
        context = MagicMock(spec=Context)
        state = ErrorState()
        state.__set_context__(context)

        with self.assertRaisesRegex(
            InvalidStateError,
            "^The connection is in an error state. Please close the connection.$",
        ):
            state.receive_data(b"some data")

    def test_close(self) -> None:
        context = MagicMock(spec=Context)
        state = ErrorState()
        state.__set_context__(context)

        state.close()

        self.assertIsInstance(
            context.__set_state__.call_args[0][0], InitialState
        )

    def test_send(self) -> None:
        context = MagicMock(spec=Context)
        state = ErrorState()
        state.__set_context__(context)

        with self.assertRaisesRegex(
            InvalidStateError,
            "^The connection is in an error state. Please close the connection.$",
        ):
            state.receive_data(b"some data")


class ReceivingDataStateTestCase(unittest.TestCase):
    def test_receive_data(self) -> None:
        context = MagicMock(spec=Context)
        request = RequestMock(b"<start/>")
        state = ReceivingDataState(request)
        state.__set_context__(context)

        response = state.receive_data(b"<response>")
        self.assertIsNone(response)

        response = state.receive_data(b"</response>")
        self.assertIsNotNone(response)
        self.assertIsInstance(response, Response)
        self.assertEqual(response.data, b"<response></response>")  # type: ignore

        self.assertIsInstance(
            context.__set_state__.call_args[0][0], InitialState
        )

    def test_close(self) -> None:
        context = MagicMock(spec=Context)
        request = RequestMock(b"<start/>")
        state = ReceivingDataState(request)
        state.__set_context__(context)

        state.close()

        self.assertIsInstance(
            context.__set_state__.call_args[0][0], InitialState
        )

    def test_send(self) -> None:
        context = MagicMock(spec=Context)
        request = RequestMock(b"<start/>")
        state = ReceivingDataState(request)
        state.__set_context__(context)

        with self.assertRaisesRegex(InvalidStateError, "Invalid State"):
            another_request = RequestMock(b"<element>")
            state.send(another_request)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ._entity_type import EntityType


class Tags:
    @classmethod
    def clone_tag(cls, tag_id: EntityID) -> Request:
        """Clone an existing tag

        Args:
            tag_id: UUID of an existing tag to clone from
        """
        if not tag_id:
            raise RequiredArgument(
                function=cls.clone_tag.__name__, argument="tag_id"
            )

        cmd = XmlCommand("create_tag")
        cmd.add_element("copy", str(tag_id))
        return cmd

    @classmethod
    def create_tag(
        cls,
        name: str,
        resource_type: EntityType,
        *,
        resource_filter: str | None = None,
        resource_ids: list[EntityID] | None = None,
        value: str | None = None,
        comment: str | None = None,
        active: bool | None = None,
    ) -> Request:
        """Create a tag

        Args:
            name: Name of the tag. A full tag name consisting of namespace and
                predicate e.g. `foo:bar`.
            resource_type: Entity type the tag is to be attached to.
            resource_filter: Filter term to select resources the tag is to be
                attached to. Only one of resource_filter or resource_ids can be
                provided.
            resource_ids: IDs of the resources the tag is to be attached to.
                Only one of resource_filter or resource_ids can be provided.
            value: Value associated with the tag.
            comment: Comment for the tag.
            active: Whether the tag should be active.
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_tag.__name__, argument="name"
            )

        if resource_filter and resource_ids:
            raise InvalidArgument(
                "create_tag accepts either resource_filter or resource_ids "
                "argument",
                function=cls.create_tag.__name__,
            )

        if not resource_type:
            raise RequiredArgument(
                function=cls.create_tag.__name__, argument="resource_type"
            )

        if not isinstance(resource_type, EntityType):
            resource_type = EntityType(resource_type)

        cmd = XmlCommand("create_tag")
        cmd.add_element("name", name)

        xml_resources = cmd.add_element("resources")
        if resource_filter is not None:
            xml_resources.set_attribute("filter", resource_filter)

        for resource_id in resource_ids or []:
            xml_resources.add_element(
                "resource", attrs={"id": str(resource_id)}
            )

        actual_resource_type = resource_type
        if resource_type.value == EntityType.AUDIT.value:
            actual_resource_type = EntityType.TASK
        elif resource_type.value == EntityType.POLICY.value:
            actual_resource_type = EntityType.SCAN_CONFIG
        xml_resources.add_element("type", actual_resource_type.value)

        if comment:
            cmd.add_element("comment", comment)

        if value:
            cmd.add_element("value", value)

        if active is not None:
            cmd.add_element("active", to_bool(active))

        return cmd

    @classmethod
    def delete_tag(
        cls, tag_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing tag

        Args:
            tag_id: UUID of the tag to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not tag_id:
            raise RequiredArgument(
                function=cls.delete_tag.__name__, argument="tag_id"
            )

        cmd = XmlCommand("delete_tag")
        cmd.set_attribute("tag_id", str(tag_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_tags(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        names_only: bool | None = None,
    ) -> Request:
        """Request a list of tags

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get tags from the trashcan instead
            names_only: Whether to get only distinct tag names
        """
        cmd = XmlCommand("get_tags")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if names_only is not None:
            cmd.set_attribute("names_only", to_bool(names_only))

        return cmd

    @classmethod
    def get_tag(cls, tag_id: EntityID) -> Request:
        """Request a single tag

        Args:
            tag_id: UUID of an existing tag
        """
        cmd = XmlCommand("get_tags")

        if not tag_id:
            raise RequiredArgument(
                function=cls.get_tag.__name__, argument="tag_id"
            )

        cmd.set_attribute("tag_id", str(tag_id))
        return cmd

    @classmethod
    def modify_tag(
        cls,
        tag_id: EntityID,
        *,
        comment: str | None = None,
        name: str | None = None,
        value: str | None = None,
        active: bool | None = None,
        resource_action: str | None = None,
        resource_type: EntityType | None = None,
        resource_filter: str | None = None,
        resource_ids: list[EntityID] | None = None,
    ) -> Request:
        """Modifies an existing tag.

        Args:
            tag_id: UUID of the tag.
            comment: Comment to add to the tag.
            name: Name of the tag.
            value: Value of the tag.
            active: Whether the tag is active.
            resource_action: Whether to add or remove resources instead of
                overwriting. One of '', 'add', 'set' or 'remove'.
            resource_type: Type of the resources to which to attach the tag.
                Required if resource_filter is set.
            resource_filter: Filter term to select resources the tag is to be
                attached to.
            resource_ids: IDs of the resources to which to attach the tag.
        """
        if not tag_id:
            raise RequiredArgument(
                function=cls.modify_tag.__name__, argument="tag_id"
            )

        cmd = XmlCommand("modify_tag")
        cmd.set_attribute("tag_id", str(tag_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if value:
            cmd.add_element("value", value)

        if active is not None:
            cmd.add_element("active", to_bool(active))

        if resource_action or resource_filter or resource_ids or resource_type:
            if resource_filter and not resource_type:
                raise RequiredArgument(
                    function=cls.modify_tag.__name__, argument="resource_type"
                )

            xml_resources = cmd.add_element("resources")
            if resource_action is not None:
                xml_resources.set_attribute("action", resource_action)

            if resource_filter is not None:
                xml_resources.set_attribute("filter", resource_filter)

            for resource_id in resource_ids or []:
                xml_resources.add_element(
                    "resource", attrs={"id": str(resource_id)}
                )

            if resource_type is not None:
                if not isinstance(resource_type, EntityType):
                    resource_type = EntityType(resource_type)

                actual_resource_type = resource_type
                if resource_type.value == EntityType.AUDIT.value:
                    actual_resource_type = EntityType.TASK
                elif resource_type.value == EntityType.POLICY.value:
                    actual_resource_type = EntityType.SCAN_CONFIG
                xml_resources.add_element("type", actual_resource_type.value)

        return cmd



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_delete_report import GmpDeleteAuditReportTestMixin
from .test_get_report import GmpGetAuditReportTestMixin
from .test_get_reports import GmpGetAuditReportsTestMixin

__all__ = (
    "GmpDeleteAuditReportTestMixin",
    "GmpGetAuditReportTestMixin",
    "GmpGetAuditReportsTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
import warnings

from gvm.utils import deprecation


class TestDeprecation(unittest.TestCase):
    def test_deprecation(self):
        # pylint: disable=invalid-name
        with warnings.catch_warnings(record=True) as w:
            self.assertEqual(len(w), 0)

            warnings.simplefilter("always")

            deprecation("I am deprecated")

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))
            self.assertIn(str(w[0].message), "I am deprecated")


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_get_user_setting import GmpGetUserSettingTestMixin
from .test_get_user_settings import GmpGetUserSettingsTestMixin
from .test_modify_user_setting import GmpModifyUserSettingTestMixin

__all__ = (
    "GmpGetUserSettingTestMixin",
    "GmpGetUserSettingsTestMixin",
    "GmpModifyUserSettingTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetPermissionsTestMixin:
    def test_get_permissions(self):
        self.gmp.get_permissions()

        self.connection.send.has_been_called_with(b"<get_permissions/>")

    def test_get_permissions_with_filter_string(self):
        self.gmp.get_permissions(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_permissions filter="foo=bar"/>'
        )

    def test_get_permissions_with_filter_id(self):
        self.gmp.get_permissions(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_permissions filt_id="f1"/>'
        )

    def test_get_permissions_with_trash(self):
        self.gmp.get_permissions(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_permissions trash="1"/>'
        )

        self.gmp.get_permissions(trash=False)

        self.connection.send.has_been_called_with(
            b'<get_permissions trash="0"/>'
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpDeleteAlertTestMixin:
    def test_delete(self):
        self.gmp.delete_alert("a1")

        self.connection.send.has_been_called_with(
            b'<delete_alert alert_id="a1" ultimate="0"/>'
        )

    def test_delete_ultimate(self):
        self.gmp.delete_alert("a1", ultimate=True)

        self.connection.send.has_been_called_with(
            b'<delete_alert alert_id="a1" ultimate="1"/>'
        )

    def test_missing_alert_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.delete_alert(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.delete_alert("")



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.policies import (
    GmpClonePolicyTestMixin,
    GmpCreatePolicyTestMixin,
    GmpDeletePolicyTestMixin,
    GmpGetPoliciesTestMixin,
    GmpGetPolicyTestMixin,
    GmpImportPolicyTestMixin,
    GmpModifyPolicySetCommentTestMixin,
    GmpModifyPolicySetFamilySelectionTestMixin,
    GmpModifyPolicySetNameTestMixin,
    GmpModifyPolicySetNvtPreferenceTestMixin,
    GmpModifyPolicySetNvtSelectionTestMixin,
    GmpModifyPolicySetScannerPreferenceTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPClonePolicyTestCase(GmpClonePolicyTestMixin, GMPTestCase):
    pass


class GMPCreatePolicyTestCase(GmpCreatePolicyTestMixin, GMPTestCase):
    pass


class GMPDeletePolicyTestCase(GmpDeletePolicyTestMixin, GMPTestCase):
    pass


class GMPGetPolicyTestCase(GmpGetPolicyTestMixin, GMPTestCase):
    pass


class GMPGetPoliciesTestCase(GmpGetPoliciesTestMixin, GMPTestCase):
    pass


class GMPImportPolicyTestCase(GmpImportPolicyTestMixin, GMPTestCase):
    pass


class GMPModifyPolicySetCommentTestCase(
    GmpModifyPolicySetCommentTestMixin, GMPTestCase
):
    pass


class GMPModifyPolicySetFamilySelectionTestCase(
    GmpModifyPolicySetFamilySelectionTestMixin, GMPTestCase
):
    pass


class GMPModifyPolicySetNvtSelectionTestCase(
    GmpModifyPolicySetNvtSelectionTestMixin, GMPTestCase
):
    pass


class GMPModifyPolicySetNameTestCase(
    GmpModifyPolicySetNameTestMixin, GMPTestCase
):
    pass


class GMPModifyPolicySetNvtPreferenceTestCase(
    GmpModifyPolicySetNvtPreferenceTestMixin, GMPTestCase
):
    pass


class GMPModifyPolicySetScannerPreferenceTestCase(
    GmpModifyPolicySetScannerPreferenceTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetTargetTestMixin:
    def test_get_target(self):
        self.gmp.get_target("t1")

        self.connection.send.has_been_called_with(
            b'<get_targets target_id="t1"/>'
        )

        self.gmp.get_target(target_id="t1")

        self.connection.send.has_been_called_with(
            b'<get_targets target_id="t1"/>'
        )

    def test_get_target_missing_target_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_target(target_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_target("")

    def test_get_target_with_tasks(self):
        self.gmp.get_target(target_id="t1", tasks=True)

        self.connection.send.has_been_called_with(
            b'<get_targets target_id="t1" tasks="1"/>'
        )

        self.gmp.get_target(target_id="t1", tasks=False)

        self.connection.send.has_been_called_with(
            b'<get_targets target_id="t1" tasks="0"/>'
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyTLSCertificateTestMixin:
    def test_modify_tls_certificate(self):
        self.gmp.modify_tls_certificate("c1")

        self.connection.send.has_been_called_with(
            b'<modify_tls_certificate tls_certificate_id="c1"/>'
        )

    def test_modify_tls_certificate_with_name(self):
        self.gmp.modify_tls_certificate("c1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_tls_certificate tls_certificate_id="c1">'
            b"<name>foo</name>"
            b"</modify_tls_certificate>"
        )

    def test_modify_tls_certificate_with_comment(self):
        self.gmp.modify_tls_certificate("c1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_tls_certificate tls_certificate_id="c1">'
            b"<comment>foo</comment>"
            b"</modify_tls_certificate>"
        )

    def test_modify_tls_certificate_with_trust(self):
        self.gmp.modify_tls_certificate("c1", trust=True)

        self.connection.send.has_been_called_with(
            b'<modify_tls_certificate tls_certificate_id="c1">'
            b"<trust>1</trust>"
            b"</modify_tls_certificate>"
        )

        self.gmp.modify_tls_certificate("c1", trust=False)

        self.connection.send.has_been_called_with(
            b'<modify_tls_certificate tls_certificate_id="c1">'
            b"<trust>0</trust>"
            b"</modify_tls_certificate>"
        )

    def test_missing_tls_certificate_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_tls_certificate(name="foo", tls_certificate_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_tls_certificate(name="foo", tls_certificate_id=None)



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.errors import RequiredArgument


class GmpGetAgentGroupTestMixin:
    def test_get_agent_group(self):
        self.gmp.get_agent_group(agent_group_id="group-1")

        self.connection.send.has_been_called_with(
            b'<get_agent_groups agent_group_id="group-1"/>'
        )

    def test_get_agent_group_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_agent_group(agent_group_id=None)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .reports import (
    GmpDeleteReportTestMixin,
    GmpGetReportsTestMixin,
    GmpGetReportTestMixin,
    GmpImportReportTestMixin,
)


class Gmpv224DeleteReportTestCase(GmpDeleteReportTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetReportTestCase(GmpGetReportTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetReportsTestCase(GmpGetReportsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ImportReportTestCase(GmpImportReportTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.errors import RequiredArgument


class GmpCloneAgentGroupTestMixin:
    def test_clone_agent_group(self):
        self.gmp.clone_agent_group(agent_group_id="group-1")

        self.connection.send.has_been_called_with(
            b"<create_agent_group><copy>group-1</copy></create_agent_group>"
        )

    def test_clone_agent_group_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.clone_agent_group(agent_group_id=None)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, InvalidArgumentType, RequiredArgument
from gvm.protocols.core import Request
from gvm.protocols.gmp.requests.v224 import (
    Aggregates,
    AggregateStatistic,
    EntityType,
    SortOrder,
)


class AggregatesTestCase(unittest.TestCase):
    def test_get_aggregates(self):
        """
        Test basic get_aggregates calls with only resource_type except special
        cases for audit, policy, scan_config and task.
        """
        request = Aggregates.get_aggregates(EntityType.ALERT)

        self.assertIsInstance(request, Request)
        self.assertEqual(bytes(request), b'<get_aggregates type="alert"/>')

        request = Aggregates.get_aggregates(
            resource_type=EntityType.CERT_BUND_ADV
        )

        self.assertEqual(
            bytes(request), b'<get_aggregates type="cert_bund_adv"/>'
        )

        request = Aggregates.get_aggregates(EntityType.CPE)

        self.assertEqual(bytes(request), b'<get_aggregates type="cpe"/>')

        request = Aggregates.get_aggregates(EntityType.CVE)

        self.assertEqual(bytes(request), b'<get_aggregates type="cve"/>')

        request = Aggregates.get_aggregates(EntityType.DFN_CERT_ADV)

        self.assertEqual(
            bytes(request), b'<get_aggregates type="dfn_cert_adv"/>'
        )

        request = Aggregates.get_aggregates(EntityType.HOST)

        self.assertEqual(bytes(request), b'<get_aggregates type="host"/>')

        request = Aggregates.get_aggregates(EntityType.NOTE)

        self.assertEqual(bytes(request), b'<get_aggregates type="note"/>')

        request = Aggregates.get_aggregates(EntityType.NVT)

        self.assertEqual(bytes(request), b'<get_aggregates type="nvt"/>')

        request = Aggregates.get_aggregates(EntityType.OPERATING_SYSTEM)

        self.assertEqual(bytes(request), b'<get_aggregates type="os"/>')

        request = Aggregates.get_aggregates(EntityType.OVALDEF)

        self.assertEqual(bytes(request), b'<get_aggregates type="ovaldef"/>')

        request = Aggregates.get_aggregates(EntityType.OVERRIDE)

        self.assertEqual(bytes(request), b'<get_aggregates type="override"/>')

        request = Aggregates.get_aggregates(EntityType.REPORT)

        self.assertEqual(bytes(request), b'<get_aggregates type="report"/>')

        request = Aggregates.get_aggregates(EntityType.RESULT)

        self.assertEqual(bytes(request), b'<get_aggregates type="result"/>')

    def test_get_aggregates_resource_types_with_usage_type(self):
        """
        Test special cases of resource_type in get_aggregates calls that
        should add a usage_type parameter: audit, policy, scan_config and task.
        """
        request = Aggregates.get_aggregates(EntityType.AUDIT)

        self.assertEqual(
            bytes(request), b'<get_aggregates usage_type="audit" type="task"/>'
        )

        request = Aggregates.get_aggregates(EntityType.POLICY)

        self.assertEqual(
            bytes(request),
            b'<get_aggregates usage_type="policy" type="config"/>',
        )

        request = Aggregates.get_aggregates(EntityType.SCAN_CONFIG)

        self.assertEqual(
            bytes(request), b'<get_aggregates usage_type="scan" type="config"/>'
        )

        request = Aggregates.get_aggregates(EntityType.TASK)

        self.assertEqual(
            bytes(request), b'<get_aggregates usage_type="scan" type="task"/>'
        )

    def test_get_aggregates_missing_resource_type(self):
        """
        Test get_aggregates calls with missing resource_type
        """
        with self.assertRaises(RequiredArgument):
            Aggregates.get_aggregates(resource_type=None)

        with self.assertRaises(RequiredArgument):
            Aggregates.get_aggregates(resource_type="")

        with self.assertRaises(RequiredArgument):
            Aggregates.get_aggregates("")

    def test_get_aggregates_invalid_resource_type(self):
        """
        Test get_aggregates calls with invalid resource_type
        """
        with self.assertRaises(InvalidArgument):
            Aggregates.get_aggregates(resource_type="foo")

    def test_get_aggregates_sort_criteria(self):
        """
        Test get_aggregates calls with sort_criteria given as strings
        """
        request = Aggregates.get_aggregates(
            EntityType.NVT,
            group_column="family",
            sort_criteria=[
                {"field": "severity", "stat": "mean", "order": "descending"},
                {"stat": "count", "order": "descending"},
                {"field": "family", "order": "ascending"},
            ],
            data_columns=["severity"],
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates type="nvt" group_column="family">'
            b'<sort field="severity" stat="mean" order="descending"/>'
            b'<sort stat="count" order="descending"/>'
            b'<sort field="family" order="ascending"/>'
            b"<data_column>severity</data_column>"
            b"</get_aggregates>",
        )

    def test_get_aggregates_sort_criteria_enum(self):
        """
        Test get_aggregates calls with sort_criteria given as enums
        """
        request = Aggregates.get_aggregates(
            EntityType.NVT,
            group_column="family",
            sort_criteria=[
                {
                    "field": "severity",
                    "stat": AggregateStatistic.MEAN,
                    "order": SortOrder.DESCENDING,
                }
            ],
            data_columns=["severity"],
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates type="nvt" group_column="family">'
            b'<sort field="severity" stat="mean" order="descending"/>'
            b"<data_column>severity</data_column>"
            b"</get_aggregates>",
        )

    def test_get_aggregates_invalid_sort_criteria(self):
        """
        Test get_aggregates calls with invalid sort_criteria
        """
        with self.assertRaises(InvalidArgumentType):
            Aggregates.get_aggregates(
                resource_type=EntityType.ALERT, sort_criteria="INVALID"
            )

        with self.assertRaises(InvalidArgumentType):
            Aggregates.get_aggregates(
                resource_type=EntityType.ALERT, sort_criteria=["INVALID"]
            )

        with self.assertRaisesRegex(
            InvalidArgument,
            "^Invalid argument INVALID. Allowed values are COUNT,C_COUNT,C_SUM,"
            "MAX,MEAN,MIN,SUM,TEXT,VALUE.$",
        ):
            Aggregates.get_aggregates(
                resource_type=EntityType.ALERT,
                sort_criteria=[{"stat": "INVALID"}],
            )

        with self.assertRaises(InvalidArgument):
            Aggregates.get_aggregates(
                resource_type=EntityType.ALERT,
                sort_criteria=[{"order": "INVALID"}],
            )

    def test_get_aggregates_group_limits(self):
        """
        Test get_aggregates calls with group limits (first_group, max_groups)
        """
        request = Aggregates.get_aggregates(
            EntityType.CPE, first_group=20, max_groups=25
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates type="cpe" first_group="20" max_groups="25"/>',
        )

    def test_get_aggregates_invalid_group_limits(self):
        """
        Test get_aggregates calls with invalid group limits
        """
        with self.assertRaises(InvalidArgumentType):
            Aggregates.get_aggregates(
                EntityType.CPE, first_group="INVALID", max_groups=25
            )

        with self.assertRaises(InvalidArgumentType):
            Aggregates.get_aggregates(
                EntityType.CPE, first_group=1, max_groups="INVALID"
            )

    def test_get_aggregates_data_columns(self):
        """
        Test get_aggregates calls with data_columns
        """
        request = Aggregates.get_aggregates(
            EntityType.CPE, data_columns=["severity", "cves"]
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates type="cpe">'
            b"<data_column>severity</data_column>"
            b"<data_column>cves</data_column>"
            b"</get_aggregates>",
        )

    def test_get_aggregates_invalid_data_columns(self):
        """
        Test get_aggregates calls with invalid data_columns
        """
        with self.assertRaises(InvalidArgumentType):
            Aggregates.get_aggregates(
                resource_type=EntityType.ALERT, data_columns=123
            )

    def test_get_aggregates_group_column(self):
        """
        Test get_aggregates calls with group_column
        """
        request = Aggregates.get_aggregates(
            EntityType.NVT, group_column="family"
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates type="nvt" group_column="family"/>',
        )

    def test_get_aggregates_subgroup_column(self):
        """
        Test get_aggregates calls with subgroup_column
        """
        request = Aggregates.get_aggregates(
            EntityType.NVT,
            group_column="family",
            subgroup_column="solution_type",
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates type="nvt" group_column="family"'
            b' subgroup_column="solution_type"/>',
        )

    def test_get_aggregates_missing_group_column(self):
        """
        Test get_aggregates calls with group_column missing
        if subgroup_column was given.
        """
        with self.assertRaises(RequiredArgument):
            Aggregates.get_aggregates(
                resource_type=EntityType.NVT, subgroup_column="solution_type"
            )

        with self.assertRaises(RequiredArgument):
            Aggregates.get_aggregates(
                resource_type=EntityType.NVT,
                group_column="",
                subgroup_column="solution_type",
            )

    def test_get_aggregates_text_columns(self):
        """
        Test get_aggregates calls with text_columns
        """
        request = Aggregates.get_aggregates(
            EntityType.SCAN_CONFIG,
            group_column="uuid",
            text_columns=["name", "comment"],
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates usage_type="scan" type="config"'
            b' group_column="uuid">'
            b"<text_column>name</text_column>"
            b"<text_column>comment</text_column>"
            b"</get_aggregates>",
        )

    def test_get_aggregates_invalid_text_columns(self):
        """
        Test get_aggregates calls with invalid text_columns
        """
        with self.assertRaises(InvalidArgumentType):
            Aggregates.get_aggregates(
                resource_type=EntityType.ALERT, text_columns=123
            )

    def test_get_aggregates_mode(self):
        """
        Test get_aggregates calls with mode
        """
        request = Aggregates.get_aggregates(
            EntityType.NVT, group_column="name", mode="word_counts"
        )

        self.assertEqual(
            bytes(request),
            b'<get_aggregates type="nvt" group_column="name"'
            b' mode="word_counts"/>',
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpnext import GMPTestCase
from ..entities.credential_stores import (
    GmpGetCredentialStoresTestMixin,
    GmpModifyCredentialStoreTestMixin,
    GmpVerifyCredentialStoreTestMixin,
)


class GMPGetCredentialStoresTest(GmpGetCredentialStoresTestMixin, GMPTestCase):
    pass


class GMPModifyCredentialStoreTest(
    GmpModifyCredentialStoreTestMixin, GMPTestCase
):
    pass


class GMPVerifyCredentialStoreTest(
    GmpVerifyCredentialStoreTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetFilterTestMixin:
    def test_get_filter(self):
        self.gmp.get_filter("f1")

        self.connection.send.has_been_called_with(
            b'<get_filters filter_id="f1"/>'
        )

        self.gmp.get_filter(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_filters filter_id="f1"/>'
        )

    def test_get_filter_missing_filter_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_filter(filter_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_filter("")

    def test_get_filter_with_alerts(self):
        self.gmp.get_filter(filter_id="f1", alerts=True)

        self.connection.send.has_been_called_with(
            b'<get_filters filter_id="f1" alerts="1"/>'
        )

        self.gmp.get_filter(filter_id="f1", alerts=False)

        self.connection.send.has_been_called_with(
            b'<get_filters filter_id="f1" alerts="0"/>'
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetAuditsTestMixin:
    def test_get_audits_simple(self):
        self.gmp.get_audits()

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="audit"/>'
        )

    def test_get_audits_with_filter_string(self):
        self.gmp.get_audits(filter_string="name=foo")

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="audit" filter="name=foo"/>'
        )

    def test_get_audits_with_filter_id(self):
        self.gmp.get_audits(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="audit" filt_id="f1"/>'
        )

    def test_get_audits_from_trash(self):
        self.gmp.get_audits(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="audit" trash="1"/>'
        )

    def test_get_audits_with_details(self):
        self.gmp.get_audits(details=True)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="audit" details="1"/>'
        )

    def test_get_audits_without_details(self):
        self.gmp.get_audits(details=False)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="audit" details="0"/>'
        )

    def test_get_audits_with_schedules_only(self):
        self.gmp.get_audits(schedules_only=True)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="audit" schedules_only="1"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_override import GmpCloneOverrideTestMixin
from .test_create_override import GmpCreateOverrideTestMixin
from .test_delete_override import GmpDeleteOverrideTestMixin
from .test_get_override import GmpGetOverrideTestMixin
from .test_get_overrides import GmpGetOverridesTestMixin
from .test_modify_override import GmpModifyOverrideTestMixin

__all__ = (
    "GmpCloneOverrideTestMixin",
    "GmpCreateOverrideTestMixin",
    "GmpDeleteOverrideTestMixin",
    "GmpGetOverrideTestMixin",
    "GmpGetOverridesTestMixin",
    "GmpModifyOverrideTestMixin",
)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_base64
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class UserSettings:
    @staticmethod
    def get_user_settings(*, filter_string: str | None = None) -> Request:
        """Request a list of user settings

        Args:
            filter_string: Filter term to use for the query
        """
        cmd = XmlCommand("get_settings")

        if filter_string:
            cmd.set_attribute("filter", filter_string)

        return cmd

    @classmethod
    def get_user_setting(cls, setting_id: EntityID) -> Request:
        """Request a single user setting

        Args:
            setting_id: UUID of an existing setting
        """
        cmd = XmlCommand("get_settings")

        if not setting_id:
            raise RequiredArgument(
                function=cls.get_user_setting.__name__, argument="setting_id"
            )

        cmd.set_attribute("setting_id", str(setting_id))
        return cmd

    @classmethod
    def modify_user_setting(
        cls,
        *,
        setting_id: EntityID | None = None,
        name: str | None = None,
        value: str | None = None,
    ) -> Request:
        """Modifies an existing user setting.

        Args:
            setting_id: UUID of the setting to be changed.
            name: The name of the setting. Either setting_id or name must be
                passed.
            value: The value of the setting.
        """
        if not setting_id and not name:
            raise RequiredArgument(
                function=cls.modify_user_setting.__name__,
                argument="setting_id or name argument",
            )

        if value is None:
            raise RequiredArgument(
                function=cls.modify_user_setting.__name__,
                argument="value argument",
            )

        cmd = XmlCommand("modify_setting")

        if setting_id:
            cmd.set_attribute("setting_id", str(setting_id))
        else:
            cmd.add_element("name", name)

        cmd.add_element("value", to_base64(value))

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCloneTargetTestMixin:
    TARGET_ID = "00000000-0000-0000-0000-000000000000"

    def test_clone(self):
        self.gmp.clone_target(self.TARGET_ID)

        self.connection.send.has_been_called_with(
            "<create_target>"
            f"<copy>{self.TARGET_ID}</copy>"
            "</create_target>".encode()
        )

    def test_missing_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.clone_target("")

        with self.assertRaises(RequiredArgument):
            self.gmp.clone_target(None)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_note import GmpCloneNoteTestMixin
from .test_create_note import GmpCreateNoteTestMixin
from .test_delete_note import GmpDeleteNoteTestMixin
from .test_get_note import GmpGetNoteTestMixin
from .test_get_notes import GmpGetNotesTestMixin
from .test_modify_note import GmpModifyNoteTestMixin

__all__ = (
    "GmpCloneNoteTestMixin",
    "GmpCreateNoteTestMixin",
    "GmpDeleteNoteTestMixin",
    "GmpGetNoteTestMixin",
    "GmpGetNotesTestMixin",
    "GmpModifyNoteTestMixin",
)



# SPDX-FileCopyrightText: 2018-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreateContainerTaskTestMixin:
    def test_create_container_task_emits_deprecation_warning_and_sends_xml(
        self,
    ):
        with self.assertWarns(DeprecationWarning):
            self.gmp.create_container_task(name="foo")

        self.connection.send.has_been_called_with(
            b'<create_task><name>foo</name><target id="0"/></create_task>'
        )

    def test_create_container_task_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_container_task(name=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_container_task(name="")

    def test_create_container_task_with_comment_emits_warning(self):
        with self.assertWarns(DeprecationWarning):
            self.gmp.create_container_task(name="foo", comment="bar")

        self.connection.send.has_been_called_with(
            b"<create_task>"
            b"<name>foo</name>"
            b'<target id="0"/>'
            b"<comment>bar</comment>"
            b"</create_task>"
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_filter import GmpCloneFilterTestMixin
from .test_create_filter import GmpCreateFilterTestMixin
from .test_delete_filter import GmpDeleteFilterTestMixin
from .test_get_filter import GmpGetFilterTestMixin
from .test_get_filters import GmpGetFiltersTestMixin
from .test_modify_filter import GmpModifyFilterTestMixin

__all__ = (
    "GmpCloneFilterTestMixin",
    "GmpCreateFilterTestMixin",
    "GmpDeleteFilterTestMixin",
    "GmpGetFilterTestMixin",
    "GmpGetFiltersTestMixin",
    "GmpModifyFilterTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetAlertsTestMixin:
    def test_get_alerts(self):
        self.gmp.get_alerts()

        self.connection.send.has_been_called_with(b"<get_alerts/>")

    def test_get_alerts_with_trash(self):
        self.gmp.get_alerts(trash=True)

        self.connection.send.has_been_called_with(b'<get_alerts trash="1"/>')

        self.gmp.get_alerts(trash=False)

        self.connection.send.has_been_called_with(b'<get_alerts trash="0"/>')

    def test_get_alerts_with_filter_string(self):
        self.gmp.get_alerts(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_alerts filter="foo=bar"/>'
        )

    def test_get_alerts_with_filter_id(self):
        self.gmp.get_alerts(filter_id="f1")

        self.connection.send.has_been_called_with(b'<get_alerts filt_id="f1"/>')

    def test_get_alerts_with_tasks(self):
        self.gmp.get_alerts(tasks=True)

        self.connection.send.has_been_called_with(b'<get_alerts tasks="1"/>')

        self.gmp.get_alerts(tasks=False)

        self.connection.send.has_been_called_with(b'<get_alerts tasks="0"/>')



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later

from .test_get_credential_stores import GmpGetCredentialStoresTestMixin
from .test_modify_credential_stores import GmpModifyCredentialStoreTestMixin
from .test_verify_credential_stores import GmpVerifyCredentialStoreTestMixin

__all__ = (
    "GmpGetCredentialStoresTestMixin",
    "GmpModifyCredentialStoreTestMixin",
    "GmpVerifyCredentialStoreTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Module for transforming responses
"""

from lxml import etree

from gvm.protocols.core import Response

from .errors import GvmError, GvmResponseError, GvmServerError
from .xml import Element, create_parser


class EtreeTransform:
    """
    Transform a response into a lxml.etree root element
    """

    def __init__(self):
        self._parser = create_parser()

    def _convert_response(self, response: Response) -> Element:
        return etree.XML(bytes(response), parser=self._parser)

    def __call__(self, response: Response) -> Element:
        return self._convert_response(response)


def check_command_status(root: Element):
    status = root.get("status")

    if status is None:
        raise GvmServerError("No status in response.", str(root))

    if status[0] == "4":
        raise GvmResponseError(status=status, message=root.get("status_text"))
    elif status[0] == "5":
        raise GvmServerError(status=status, message=root.get("status_text"))
    elif status[0] != "2":
        raise GvmError(f"Error in response. {root.get('status_text')}", root)


class CheckCommandTransform(EtreeTransform):
    """
    Check the response code of a response and raise GmpError if
    response was an error response
    """

    def __call__(self, response: Response) -> Response:  # type: ignore[override]
        root = self._convert_response(response)

        check_command_status(root)

        return response


class EtreeCheckCommandTransform(EtreeTransform):
    """
    Transform a response into a lxml.etree root element and raise GmpError if
    response was an error response
    """

    def __call__(self, response: Response) -> Element:
        root = self._convert_response(response)

        check_command_status(root)

        return root



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class Results:
    @classmethod
    def get_result(cls, result_id: EntityID) -> Request:
        """Request a single result

        Args:
            result_id: UUID of an existing result
        """
        cmd = XmlCommand("get_results")

        if not result_id:
            raise RequiredArgument(
                function=cls.get_result.__name__, argument="result_id"
            )

        cmd.set_attribute("result_id", str(result_id))

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd

    @staticmethod
    def get_results(
        *,
        filter_string: str | None = None,
        filter_id: str | None = None,
        task_id: str | None = None,
        note_details: bool | None = None,
        override_details: bool | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of results

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            task_id: UUID of task for note and override handling
            note_details: If notes are included, whether to include note details
            override_details: If overrides are included, whether to include
                override details
            details: Whether to include additional details of the results
        """
        cmd = XmlCommand("get_results")

        cmd.add_filter(filter_string, filter_id)

        if task_id:
            cmd.set_attribute("task_id", task_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if note_details is not None:
            cmd.set_attribute("note_details", to_bool(note_details))

        if override_details is not None:
            cmd.set_attribute("override_details", to_bool(override_details))

        return cmd



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.utils import add_filter
from gvm.xml import XmlCommand


class TestAddFilter(unittest.TestCase):
    def test_add_filter(self):
        cmd = XmlCommand("test")
        filter_string = "foo"

        add_filter(cmd, filter_string=filter_string, filter_id=None)

        self.assertEqual(cmd.to_string(), '<test filter="foo"/>')

    def test_add_filter_id(self):
        cmd = XmlCommand("test")
        filter_id = "foo"

        add_filter(cmd, filter_string=None, filter_id=filter_id)

        self.assertEqual(cmd.to_string(), '<test filt_id="foo"/>')


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import CertBundAdvisories


class CertBundAdvisoriesTestCase(unittest.TestCase):
    def test_get_cert_bund_advisories(self):
        request = CertBundAdvisories.get_cert_bund_advisories()
        self.assertEqual(
            bytes(request),
            b'<get_info type="CERT_BUND_ADV"/>',
        )

    def test_get_cert_bund_advisories_with_filter_string(self):
        request = CertBundAdvisories.get_cert_bund_advisories(
            filter_string="filter_string"
        )
        self.assertEqual(
            bytes(request),
            b'<get_info type="CERT_BUND_ADV" filter="filter_string"/>',
        )

    def test_get_cert_bund_advisories_with_filter_id(self):
        request = CertBundAdvisories.get_cert_bund_advisories(
            filter_id="filter_id"
        )
        self.assertEqual(
            bytes(request),
            b'<get_info type="CERT_BUND_ADV" filt_id="filter_id"/>',
        )

    def test_get_cert_bund_advisories_with_name(self):
        request = CertBundAdvisories.get_cert_bund_advisories(name="name")
        self.assertEqual(
            bytes(request),
            b'<get_info type="CERT_BUND_ADV" name="name"/>',
        )

    def test_get_cert_bund_advisories_with_details(self):
        request = CertBundAdvisories.get_cert_bund_advisories(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_info type="CERT_BUND_ADV" details="1"/>',
        )

        request = CertBundAdvisories.get_cert_bund_advisories(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_info type="CERT_BUND_ADV" details="0"/>',
        )

    def test_get_cert_bund_advisory(self):
        request = CertBundAdvisories.get_cert_bund_advisory("cert_id")
        self.assertEqual(
            bytes(request),
            b'<get_info info_id="cert_id" type="CERT_BUND_ADV" details="1"/>',
        )

    def test_get_cert_bund_advisory_missing_cert_id(self):
        with self.assertRaises(RequiredArgument):
            CertBundAdvisories.get_cert_bund_advisory(None)

        with self.assertRaises(RequiredArgument):
            CertBundAdvisories.get_cert_bund_advisory("")



# SPDX-FileCopyrightText: 2019-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from unittest.mock import MagicMock, patch

from gvm.errors import GvmError
from gvm.protocols.gmp import Gmp, GMPNext
from gvm.protocols.gmp._gmp224 import GMPv224
from gvm.protocols.gmp._gmp225 import GMPv225
from gvm.protocols.gmp._gmp226 import GMPv226
from gvm.protocols.gmp._gmp227 import GMPv227
from tests.protocols import GmpTestCase


class GmpContextManagerTestCase(GmpTestCase):
    gmp_class = Gmp

    def test_select_gmpv7(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>7.0</version>"
            b"</get_version_response>"
        )

        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    def test_select_gmpv8(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>8.0</version>"
            b"</get_version_response>"
        )

        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    def test_select_gmpv9(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>9.0</version>"
            b"</get_version_response>"
        )

        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    def test_select_gmpv208(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>20.08</version>"
            b"</get_version_response>"
        )
        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    def test_select_gmpv214(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>21.04</version>"
            b"</get_version_response>"
        )

        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    def test_select_gmpv224(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.04</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 4))
            self.assertIsInstance(gmp, GMPv224)

    def test_select_gmpv225(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.05</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 5))
            self.assertIsInstance(gmp, GMPv225)

        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.5</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 5))
            self.assertIsInstance(gmp, GMPv225)

    def test_select_gmpv226(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.06</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 6))
            self.assertIsInstance(gmp, GMPv226)

        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.6</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 6))
            self.assertIsInstance(gmp, GMPv226)

    def test_select_gmpv227(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.07</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 7))
            self.assertIsInstance(gmp, GMPv227)

        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.7</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 7))
            self.assertIsInstance(gmp, GMPv227)

    def test_select_gmpv228(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.08</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 8))
            self.assertIsInstance(gmp, GMPNext)

        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.8</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 8))
            self.assertIsInstance(gmp, GMPv227)

    def test_next_version_fallback(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.9</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 8))
            self.assertIsInstance(gmp, GMPv227)

    def test_newer_version_fallback(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.20</version>"
            b"</get_version_response>"
        )

        with self.gmp as gmp:
            self.assertEqual(gmp.get_protocol_version(), (22, 8))
            self.assertIsInstance(gmp, GMPv227)

    def test_unknown_protocol(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>1.0</version>"
            b"</get_version_response>"
        )

        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    def test_missing_version_in_response(self):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<foo>bar</foo>"
            b"</get_version_response>"
        )

        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    def test_invalid_response(self):
        self.connection.read.return_value(b"<get_foo_response/>")

        with self.assertRaises(GvmError):
            with self.gmp:
                pass

    @patch("gvm.protocols.gmp._gmp.GMPv224", autospec=True)
    def test_connect_disconnect(self, gmp_mock: MagicMock):
        self.connection.read.return_value(
            b'<get_version_response status="200" status_text="OK">'
            b"<version>22.04</version>"
            b"</get_version_response>"
        )

        with self.gmp:
            gmp_mock.assert_called_once()

        mock_instance = gmp_mock.return_value
        mock_instance.connect.assert_called_once()
        mock_instance.disconnect.assert_called_once()


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from collections import OrderedDict

from gvm.errors import RequiredArgument
from gvm.protocols.ospv1 import Osp

from .. import MockConnection


class OSPStartScanTestCase(unittest.TestCase):
    def setUp(self):
        self.connection = MockConnection()
        self.osp = Osp(self.connection)

    def test_start_scan(self):
        scanner_params = OrderedDict()
        scanner_params["key1"] = "value1"

        targets = []
        _target1 = OrderedDict()
        _target1["hosts"] = "localhost"
        _target1["ports"] = "22,80"
        targets.append(_target1)

        _target2 = OrderedDict()
        _target2["hosts"] = "192.168.10.1"
        _target2["ports"] = "443"
        _smb = OrderedDict()
        _smb["username"] = "username"
        _smb["password"] = "pass"
        _smb["port"] = "port"
        _smb["type"] = "type"
        _credential1 = {"smb": _smb}
        _target2["credentials"] = _credential1
        targets.append(_target2)

        vts = OrderedDict()
        vts["vt1"] = {"value_id": "value"}
        vts["vt_groups"] = ["family=A", "family=B"]

        self.osp.start_scan(
            scan_id="123-456",
            parallel=10,
            scanner_params=scanner_params,
            targets=targets,
            vt_selection=vts,
        )

        self.connection.send.has_been_called_with(
            b'<start_scan scan_id="123-456" parallel="10">'
            b'<scanner_params key1="value1"/>'
            b"<targets><target><hosts>localhost</hosts>"
            b"<ports>22,80</ports></target>"
            b"<target><hosts>192.168.10.1</hosts>"
            b"<ports>443</ports>"
            b"<credentials>"
            b'<credential type="type" port="port" service="smb">'
            b"<username>username</username>"
            b"<password>pass</password>"
            b"</credential></credentials>"
            b"</target></targets><vt_selection>"
            b'<vt_single id="vt1">'
            b'<vt_value id="value_id">value</vt_value></vt_single>'
            b'<vt_group filter="family=A"/><vt_group filter="family=B"/>'
            b"</vt_selection></start_scan>"
        )

    def test_start_scan_without_target(self):
        with self.assertRaises(RequiredArgument):
            self.osp.start_scan()

    def test_start_scan_legacy(self):
        self.osp.start_scan(
            scan_id="123-456", parallel=10, target="localhost", ports="22"
        )
        self.connection.send.has_been_called_with(
            b'<start_scan scan_id="123-456" parallel="10" '
            b'target="localhost" ports="22">'
            b"<scanner_params/></start_scan>"
        )


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
API wrapper for accessing the /health endpoints of the openvasd HTTP API.
"""

import httpx

from ._api import OpenvasdAPI


class HealthAPI(OpenvasdAPI):
    """
    Provides access to the openvasd /health endpoints, which expose the
    operational state of the scanner.

    All methods return the HTTP status code of the response and raise an exception
    if the server returns an error response (4xx or 5xx).
    """

    def get_alive(self) -> int:
        """
        Check if the scanner process is alive.

        Returns:
            HTTP status code (e.g., 200 if alive).

        Raises:
            httpx.HTTPStatusError: If the server response indicates failure and
                exceptions are not suppressed.

        See: GET /health/alive in the openvasd API documentation.
        """
        try:
            response = self._client.get("/health/alive")
            response.raise_for_status()
            return response.status_code
        except httpx.HTTPStatusError as e:
            if self._suppress_exceptions:
                return e.response.status_code
            raise

    def get_ready(self) -> int:
        """
        Check if the scanner is ready to accept requests (e.g., feed loaded).

        Returns:
            HTTP status code (e.g., 200 if ready).

        Raises:
            httpx.HTTPStatusError: If the server response indicates failure and
                exceptions are not suppressed.

        See: GET /health/ready in the openvasd API documentation.
        """
        try:
            response = self._client.get("/health/ready")
            response.raise_for_status()
            return response.status_code
        except httpx.HTTPStatusError as e:
            if self._suppress_exceptions:
                return e.response.status_code
            raise

    def get_started(self) -> int:
        """
        Check if the scanner has fully started.

        Returns:
            HTTP status code (e.g., 200 if started).

        Raises:
            httpx.HTTPStatusError: If the server response indicates failure and
                exceptions are not suppressed.

        See: GET /health/started in the openvasd API documentation.
        """
        try:
            response = self._client.get("/health/started")
            response.raise_for_status()
            return response.status_code
        except httpx.HTTPStatusError as e:
            if self._suppress_exceptions:
                return e.response.status_code
            raise



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import Tickets, TicketStatus


class TicketsTestCase(unittest.TestCase):
    def test_clone_ticket(self):
        request = Tickets.clone_ticket("ticket_id")
        self.assertEqual(
            bytes(request),
            b"<create_ticket><copy>ticket_id</copy></create_ticket>",
        )

    def test_clone_ticket_missing_ticket_id(self):
        with self.assertRaises(RequiredArgument):
            Tickets.clone_ticket(None)

        with self.assertRaises(RequiredArgument):
            Tickets.clone_ticket("")

    def test_create_ticket(self):
        request = Tickets.create_ticket(
            result_id="result_id",
            assigned_to_user_id="user_id",
            note="note",
        )
        self.assertEqual(
            bytes(request),
            b"<create_ticket>"
            b'<result id="result_id"/>'
            b'<assigned_to><user id="user_id"/></assigned_to>'
            b"<open_note>note</open_note>"
            b"</create_ticket>",
        )

    def test_create_ticket_with_comment(self):
        request = Tickets.create_ticket(
            result_id="result_id",
            assigned_to_user_id="user_id",
            note="note",
            comment="comment",
        )
        self.assertEqual(
            bytes(request),
            b"<create_ticket>"
            b'<result id="result_id"/>'
            b'<assigned_to><user id="user_id"/></assigned_to>'
            b"<open_note>note</open_note>"
            b"<comment>comment</comment>"
            b"</create_ticket>",
        )

    def test_create_ticket_missing_result_id(self):
        with self.assertRaises(RequiredArgument):
            Tickets.create_ticket(
                result_id=None, assigned_to_user_id="user_id", note="note"
            )
        with self.assertRaises(RequiredArgument):
            Tickets.create_ticket(
                result_id="", assigned_to_user_id="user_id", note="note"
            )

    def test_create_ticket_missing_assigned_to_user_id(self):
        with self.assertRaises(RequiredArgument):
            Tickets.create_ticket(
                result_id="result_id", assigned_to_user_id=None, note="note"
            )
        with self.assertRaises(RequiredArgument):
            Tickets.create_ticket(
                result_id="result_id", assigned_to_user_id="", note="note"
            )

    def test_create_ticket_missing_note(self):
        with self.assertRaises(RequiredArgument):
            Tickets.create_ticket(
                result_id="result_id", assigned_to_user_id="user_id", note=None
            )

        with self.assertRaises(RequiredArgument):
            Tickets.create_ticket(
                result_id="result_id", assigned_to_user_id="user_id", note=""
            )

    def test_delete_ticket(self):
        request = Tickets.delete_ticket("ticket_id")
        self.assertEqual(
            bytes(request),
            b'<delete_ticket ticket_id="ticket_id" ultimate="0"/>',
        )

    def test_delete_ticket_with_ultimate(self):
        request = Tickets.delete_ticket("ticket_id", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_ticket ticket_id="ticket_id" ultimate="1"/>',
        )

        request = Tickets.delete_ticket("ticket_id", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_ticket ticket_id="ticket_id" ultimate="0"/>',
        )

    def test_delete_ticket_missing_ticket_id(self):
        with self.assertRaises(RequiredArgument):
            Tickets.delete_ticket(None)

        with self.assertRaises(RequiredArgument):
            Tickets.delete_ticket("")

    def test_get_tickets(self):
        request = Tickets.get_tickets()
        self.assertEqual(
            bytes(request),
            b"<get_tickets/>",
        )

    def test_get_tickets_with_filter_string(self):
        request = Tickets.get_tickets(filter_string="filter")
        self.assertEqual(
            bytes(request),
            b'<get_tickets filter="filter"/>',
        )

    def test_get_tickets_with_filter_id(self):
        request = Tickets.get_tickets(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_tickets filt_id="filter_id"/>',
        )

    def test_get_tickets_with_trash(self):
        request = Tickets.get_tickets(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_tickets trash="1"/>',
        )

        request = Tickets.get_tickets(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_tickets trash="0"/>',
        )

    def test_get_ticket(self):
        request = Tickets.get_ticket("ticket_id")
        self.assertEqual(
            bytes(request),
            b'<get_tickets ticket_id="ticket_id"/>',
        )

    def test_get_ticket_missing_ticket_id(self):
        with self.assertRaises(RequiredArgument):
            Tickets.get_ticket(None)

        with self.assertRaises(RequiredArgument):
            Tickets.get_ticket("")

    def test_modify_ticket(self):
        request = Tickets.modify_ticket("ticket_id")
        self.assertEqual(
            bytes(request),
            b'<modify_ticket ticket_id="ticket_id"/>',
        )

    def test_modify_ticket_with_status_and_note(self):
        request = Tickets.modify_ticket(
            "ticket_id", status=TicketStatus.FIXED, note="note"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_ticket ticket_id="ticket_id">'
            b"<status>Fixed</status>"
            b"<fixed_note>note</fixed_note>"
            b"</modify_ticket>",
        )

        request = Tickets.modify_ticket(
            "ticket_id", status="fixed", note="note"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_ticket ticket_id="ticket_id">'
            b"<status>Fixed</status>"
            b"<fixed_note>note</fixed_note>"
            b"</modify_ticket>",
        )

    def test_modify_ticket_with_assigned_to_user_id(self):
        request = Tickets.modify_ticket(
            "ticket_id", assigned_to_user_id="user_id"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_ticket ticket_id="ticket_id">'
            b'<assigned_to><user id="user_id"/></assigned_to>'
            b"</modify_ticket>",
        )

    def test_modify_ticket_with_comment(self):
        request = Tickets.modify_ticket("ticket_id", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_ticket ticket_id="ticket_id">'
            b"<comment>comment</comment>"
            b"</modify_ticket>",
        )

    def test_modify_ticket_missing_ticket_id(self):
        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket(None)

        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket("")

    def test_modify_ticket_with_status_missing_note(self):
        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket("ticket_id", status=TicketStatus.FIXED)

        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket(
                "ticket_id", status=TicketStatus.FIXED, note=None
            )

        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket(
                "ticket_id", status=TicketStatus.FIXED, note=""
            )

    def test_modify_ticket_with_note_missing_status(self):
        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket("ticket_id", note="note")

        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket("ticket_id", note="note", status=None)

        with self.assertRaises(RequiredArgument):
            Tickets.modify_ticket("ticket_id", note="note", status="")

    def test_modify_ticket_invalid_status(self):
        with self.assertRaises(InvalidArgument):
            Tickets.modify_ticket("ticket_id", note="note", status="invalid")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import ScannerType


class GmpModifyScannerTestMixin:
    def test_modify_scanner(self):
        self.gmp.modify_scanner(scanner_id="s1")

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1"/>'
        )

    def test_modify_scanner_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scanner(scanner_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scanner(scanner_id="")

    def test_modify_scanner_with_comment(self):
        self.gmp.modify_scanner(scanner_id="s1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1">'
            b"<comment>foo</comment>"
            b"</modify_scanner>"
        )

    def test_modify_scanner_with_host(self):
        self.gmp.modify_scanner(scanner_id="s1", host="foo")

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1"><host>foo</host></modify_scanner>'
        )

    def test_modify_scanner_with_port(self):
        self.gmp.modify_scanner(scanner_id="s1", port=1234)

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1">'
            b"<port>1234</port>"
            b"</modify_scanner>"
        )

        self.gmp.modify_scanner(scanner_id="s1", port="1234")

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1">'
            b"<port>1234</port>"
            b"</modify_scanner>"
        )

    def test_modify_scanner_with_name(self):
        self.gmp.modify_scanner(scanner_id="s1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1"><name>foo</name></modify_scanner>'
        )

    def test_modify_scanner_with_ca_pub(self):
        self.gmp.modify_scanner(scanner_id="s1", ca_pub="foo")

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1">'
            b"<ca_pub>foo</ca_pub>"
            b"</modify_scanner>"
        )

    def test_modify_scanner_with_credential_id(self):
        self.gmp.modify_scanner(scanner_id="s1", credential_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1">'
            b'<credential id="c1"/>'
            b"</modify_scanner>"
        )

    def test_modify_scanner_with_scanner_type(self):
        self.gmp.modify_scanner(
            scanner_id="s1", scanner_type=ScannerType.OPENVAS_SCANNER_TYPE
        )

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1"><type>2</type></modify_scanner>'
        )

        self.gmp.modify_scanner(
            scanner_id="s1", scanner_type=ScannerType.CVE_SCANNER_TYPE
        )

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1"><type>3</type></modify_scanner>'
        )

        self.gmp.modify_scanner(
            scanner_id="s1",
            scanner_type=ScannerType.GREENBONE_SENSOR_SCANNER_TYPE,
        )

        self.connection.send.has_been_called_with(
            b'<modify_scanner scanner_id="s1"><type>5</type></modify_scanner>'
        )

    def test_modify_scanner_invalid_scanner_type(self):
        with self.assertRaises(ValueError):
            self.gmp.modify_scanner(scanner_id="s1", scanner_type="")

        with self.assertRaises(InvalidArgument):
            self.gmp.modify_scanner(scanner_id="s1", scanner_type="-1")

        with self.assertRaises(InvalidArgument):
            self.gmp.modify_scanner(scanner_id="s1", scanner_type=1)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetGroupTestMixin:
    def test_get_group(self):
        self.gmp.get_group("f1")

        self.connection.send.has_been_called_with(
            b'<get_groups group_id="f1"/>'
        )

        self.gmp.get_group(group_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_groups group_id="f1"/>'
        )

    def test_get_group_missing_group_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_group(group_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_group("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.notes import (
    GmpCloneNoteTestMixin,
    GmpCreateNoteTestMixin,
    GmpDeleteNoteTestMixin,
    GmpGetNotesTestMixin,
    GmpGetNoteTestMixin,
    GmpModifyNoteTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteNoteTestCase(GmpDeleteNoteTestMixin, GMPTestCase):
    pass


class Gmpv225GetNoteTestCase(GmpGetNoteTestMixin, GMPTestCase):
    pass


class Gmpv225GetNotesTestCase(GmpGetNotesTestMixin, GMPTestCase):
    pass


class Gmpv225CloneNoteTestCase(GmpCloneNoteTestMixin, GMPTestCase):
    pass


class Gmpv225CreateNoteTestCase(GmpCreateNoteTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyNoteTestCase(GmpModifyNoteTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class TLSCertificates:
    @classmethod
    def clone_tls_certificate(cls, tls_certificate_id: EntityID) -> Request:
        """Modifies an existing TLS certificate.

        Args:
            tls_certificate_id: The UUID of an existing TLS certificate
        """
        if not tls_certificate_id:
            raise RequiredArgument(
                function=cls.clone_tls_certificate.__name__,
                argument="tls_certificate_id",
            )

        cmd = XmlCommand("create_tls_certificate")

        cmd.add_element("copy", str(tls_certificate_id))

        return cmd

    @classmethod
    def create_tls_certificate(
        cls,
        name: str,
        certificate: str,
        *,
        comment: str | None = None,
        trust: bool | None = None,
    ) -> Request:
        """Create a new TLS certificate

        Args:
            name: Name of the TLS certificate, defaulting to the MD5
                fingerprint.
            certificate: The Base64 encoded certificate data (x.509 DER or PEM).
            comment: Comment for the TLS certificate.
            trust: Whether the certificate is trusted.
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_tls_certificate.__name__, argument="name"
            )
        if not certificate:
            raise RequiredArgument(
                function=cls.create_tls_certificate.__name__,
                argument="certificate",
            )

        cmd = XmlCommand("create_tls_certificate")

        cmd.add_element("name", name)
        cmd.add_element("certificate", certificate)

        if comment:
            cmd.add_element("comment", comment)

        if trust is not None:
            cmd.add_element("trust", to_bool(trust))

        return cmd

    @classmethod
    def delete_tls_certificate(cls, tls_certificate_id: EntityID) -> Request:
        """Deletes an existing tls certificate

        Args:
            tls_certificate_id: UUID of the tls certificate to be deleted.
        """
        if not tls_certificate_id:
            raise RequiredArgument(
                function=cls.delete_tls_certificate.__name__,
                argument="tls_certificate_id",
            )

        cmd = XmlCommand("delete_tls_certificate")
        cmd.set_attribute("tls_certificate_id", str(tls_certificate_id))
        return cmd

    @staticmethod
    def get_tls_certificates(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        include_certificate_data: bool | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of TLS certificates

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            include_certificate_data: Whether to include the certificate data in
                the response
            details: Whether to include additional details of the
                tls certificates
        """
        cmd = XmlCommand("get_tls_certificates")

        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if include_certificate_data is not None:
            cmd.set_attribute(
                "include_certificate_data", to_bool(include_certificate_data)
            )

        return cmd

    @classmethod
    def get_tls_certificate(cls, tls_certificate_id: EntityID) -> Request:
        """Request a single TLS certificate

        Args:
            tls_certificate_id: UUID of an existing TLS certificate
        """
        cmd = XmlCommand("get_tls_certificates")

        if not tls_certificate_id:
            raise RequiredArgument(
                function=cls.get_tls_certificate.__name__,
                argument="tls_certificate_id",
            )

        cmd.set_attribute("tls_certificate_id", str(tls_certificate_id))

        # for single tls certificate always request cert data
        cmd.set_attribute("include_certificate_data", "1")

        # for single entity always request all details
        cmd.set_attribute("details", "1")

        return cmd

    @classmethod
    def modify_tls_certificate(
        cls,
        tls_certificate_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        trust: bool | None = None,
    ) -> Request:
        """Modifies an existing TLS certificate.

        Args:
            tls_certificate_id: UUID of the TLS certificate to be modified.
            name: Name of the TLS certificate, defaulting to the MD5 fingerprint
            comment: Comment for the TLS certificate.
            trust: Whether the certificate is trusted.
        """
        if not tls_certificate_id:
            raise RequiredArgument(
                function=cls.modify_tls_certificate.__name__,
                argument="tls_certificate_id",
            )

        cmd = XmlCommand("modify_tls_certificate")
        cmd.set_attribute("tls_certificate_id", str(tls_certificate_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if trust is not None:
            cmd.add_element("trust", to_bool(trust))

        return cmd



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from collections.abc import Mapping, Sequence
from typing import Any

from gvm.errors import InvalidArgumentType, RequiredArgument
from gvm.protocols.core import Request
from gvm.protocols.gmp.requests._entity_id import EntityID
from gvm.utils import to_bool
from gvm.xml import XmlCommand


class Agents:
    @staticmethod
    def _add_element(element, name: str, value: Any) -> None:
        """
        Helper to add a sub-element with a value if the value is not None.

        Args:
            element: The XML parent element to which the new element is added.
            name: Name of the sub-element to create.
            value: Value to set as the text of the sub-element. If None, the
                element will not be created.
        """
        if value is None:
            return
        if isinstance(value, bool):
            value = to_bool(value)
        else:
            value = str(value)

        element.add_element(name, value)

    @classmethod
    def _validate_agent_config(
        cls, config: Mapping[str, Any], *, caller: str
    ) -> None:
        """Ensure all required fields exist, are well-shaped, and non-empty."""

        def valid_map(d: Any, key: str, path: str) -> Mapping[str, Any]:
            if not isinstance(d, Mapping):
                raise RequiredArgument(
                    function=caller, argument=f"config.{path.rstrip('.')}"
                )
            v = d.get(key)
            if not isinstance(v, Mapping):
                raise RequiredArgument(
                    function=caller, argument=f"config.{path}{key}"
                )
            return v

        def valid_value(d: Mapping[str, Any], key: str, path: str) -> Any:
            v = d.get(key)
            if v is None or (isinstance(v, str) and v.strip() == ""):
                raise RequiredArgument(
                    function=caller, argument=f"config.{path}{key}"
                )
            return v

        # agent_control.retry
        ac = valid_map(config, "agent_control", "")
        retry = valid_map(ac, "retry", "agent_control.")
        valid_value(retry, "attempts", "agent_control.retry.")
        valid_value(retry, "delay_in_seconds", "agent_control.retry.")
        valid_value(retry, "max_jitter_in_seconds", "agent_control.retry.")

        # agent_script_executor
        se = valid_map(config, "agent_script_executor", "")
        valid_value(se, "bulk_size", "agent_script_executor.")
        valid_value(se, "bulk_throttle_time_in_ms", "agent_script_executor.")
        valid_value(se, "indexer_dir_depth", "agent_script_executor.")

        sched = se.get("scheduler_cron_time")
        if isinstance(sched, str):
            items = [sched]
        elif isinstance(sched, Sequence) and not isinstance(
            sched, (str, bytes)
        ):
            items = [str(x) for x in sched]
        else:
            items = []
        if not items or any(not str(x).strip() for x in items):
            raise RequiredArgument(
                function=caller,
                argument="config.agent_script_executor.scheduler_cron_time",
            )

        # heartbeat
        hb = valid_map(config, "heartbeat", "")
        valid_value(hb, "interval_in_seconds", "heartbeat.")
        valid_value(hb, "miss_until_inactive", "heartbeat.")

    @classmethod
    def _validate_config_defaults(
        cls, config_defaults: Mapping[str, Any], *, caller: str
    ) -> None:
        """Ensure agent config defaults structure is valid."""

        def valid_map(d: Any, key: str, path: str) -> Mapping[str, Any]:
            if not isinstance(d, Mapping):
                raise RequiredArgument(
                    function=caller,
                    argument=path.rstrip("."),
                )
            v = d.get(key)
            if not isinstance(v, Mapping):
                raise RequiredArgument(
                    function=caller,
                    argument=f"{path}{key}",
                )
            return v

        def valid_bool(d: Mapping[str, Any], key: str, path: str) -> bool:
            v = d.get(key)
            if not isinstance(v, bool):
                raise InvalidArgumentType(
                    function=caller,
                    argument=f"{path}{key}",
                    arg_type="bool",
                )
            return v

        agent_defaults = valid_map(
            config_defaults, "agent_defaults", "config_defaults."
        )
        cls._validate_agent_config(agent_defaults, caller=caller)

        agent_control_defaults = valid_map(
            config_defaults,
            "agent_control_defaults",
            "config_defaults.",
        )
        valid_bool(
            agent_control_defaults,
            "update_to_latest",
            "config_defaults.agent_control_defaults.",
        )

    @classmethod
    def _append_agent_config(
        cls,
        parent,
        config: Mapping[str, Any],
        wrapper_tag: str | None = "config",
    ) -> None:
        """
        Append an agent configuration block to the given XML parent element.

        Expected config structure::

            {
                "agent_control": {
                    "retry": {
                        "attempts": 6,
                        "delay_in_seconds": 60,
                        "max_jitter_in_seconds": 10,
                    }
                },
                "agent_script_executor": {
                    "bulk_size": 2,
                    "bulk_throttle_time_in_ms": 300,
                    "indexer_dir_depth": 100,
                    "scheduler_cron_time": ["0 */12 * * *"],
                },
                "heartbeat": {
                    "interval_in_seconds": 300,
                    "miss_until_inactive": 1,
                },
            }

        Args:
            parent: The XML parent element to which the wrapper element
                should be appended.
            config: Mapping containing the agent configuration fields to
                serialize.
            wrapper_tag: Optional wrapper element name. If None, fields are
                appended directly to parent.
        """
        xml_config = (
            parent.add_element(wrapper_tag)
            if wrapper_tag is not None
            else parent
        )

        # agent_control.retry
        ac = config["agent_control"]
        retry = ac["retry"]
        xml_ac = xml_config.add_element("agent_control")
        xml_retry = xml_ac.add_element("retry")
        cls._add_element(xml_retry, "attempts", retry.get("attempts"))
        cls._add_element(
            xml_retry, "delay_in_seconds", retry.get("delay_in_seconds")
        )
        cls._add_element(
            xml_retry,
            "max_jitter_in_seconds",
            retry.get("max_jitter_in_seconds"),
        )

        # agent_script_executor
        se = config["agent_script_executor"]
        xml_se = xml_config.add_element("agent_script_executor")
        cls._add_element(xml_se, "bulk_size", se.get("bulk_size"))
        cls._add_element(
            xml_se,
            "bulk_throttle_time_in_ms",
            se.get("bulk_throttle_time_in_ms"),
        )
        cls._add_element(
            xml_se, "indexer_dir_depth", se.get("indexer_dir_depth")
        )
        sched = se.get("scheduler_cron_time")
        if isinstance(sched, str):
            sched_items = [sched]
        else:
            sched_items = list(sched or [])

        if sched_items:
            xml_sched = xml_se.add_element(
                "scheduler_cron_time",
                attrs={"is_list": "1"},
            )
            for item in sched_items:
                xml_sched.add_element("item", str(item))

        # heartbeat
        hb = config["heartbeat"]
        xml_hb = xml_config.add_element("heartbeat")
        cls._add_element(
            xml_hb, "interval_in_seconds", hb.get("interval_in_seconds")
        )
        cls._add_element(
            xml_hb, "miss_until_inactive", hb.get("miss_until_inactive")
        )

    @classmethod
    def _append_config_defaults(
        cls, parent, config_defaults: Mapping[str, Any]
    ) -> None:
        xml_defaults = parent.add_element("config_defaults")

        cls._append_agent_config(
            xml_defaults,
            config_defaults["agent_defaults"],
            wrapper_tag="agent_defaults",
        )

        control_defaults = config_defaults.get("agent_control_defaults")
        if control_defaults:
            xml_control_defaults = xml_defaults.add_element(
                "agent_control_defaults"
            )
            cls._add_element(
                xml_control_defaults,
                "update_to_latest",
                control_defaults.get("update_to_latest"),
            )

    @classmethod
    def get_agents(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of agents.

        Args:
            filter_string: Filter term to use for the query.
            filter_id: UUID of an existing filter to use for the query.
            details: Whether to include detailed agent info.
        """
        cmd = XmlCommand("get_agents")
        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def modify_agents(
        cls,
        agent_ids: list[EntityID],
        *,
        authorized: bool | None = None,
        update_to_latest: bool | None = None,
        config: Mapping[str, Any] | None = None,
        comment: str | None = None,
    ) -> Request:
        """
        Modify multiple agents.

        Args:
            agent_ids: List of agent UUIDs to modify.
            authorized: Whether the agent is authorized.
            update_to_latest: Whether the agent is allowed to update to latest automatically.
            config: Nested config, e.g.:
                {
                  "agent_control": {
                    "retry": {
                      "attempts": 6,
                      "delay_in_seconds": 60,
                      "max_jitter_in_seconds": 10,
                    }
                  },
                  "agent_script_executor": {
                      "bulk_size": 2,
                      "bulk_throttle_time_in_ms": 300,
                      "indexer_dir_depth": 100,
                      "scheduler_cron_time": ["0 */12 * * *"],  # str or list[str]
                  },
                  "heartbeat": {
                      "interval_in_seconds": 300,
                      "miss_until_inactive": 1,
                  },
                }
            comment: Optional comment for the change.
        """
        if not agent_ids:
            raise RequiredArgument(
                function=cls.modify_agents.__name__, argument="agent_ids"
            )

        cmd = XmlCommand("modify_agent")
        xml_agents = cmd.add_element("agents")

        for agent_id in agent_ids:
            xml_agents.add_element("agent", attrs={"id": agent_id})

        if authorized is not None:
            cmd.add_element("authorized", to_bool(authorized))

        if update_to_latest is not None:
            cmd.add_element("update_to_latest", to_bool(update_to_latest))

        if config is not None:
            cls._validate_agent_config(
                config, caller=cls.modify_agents.__name__
            )
            cls._append_agent_config(cmd, config)

        if comment:
            cmd.add_element("comment", comment)

        return cmd

    @classmethod
    def delete_agents(cls, agent_ids: list[EntityID]) -> Request:
        """Delete multiple agents

        Args:
            agent_ids: List of agent UUIDs to delete
        """
        if not agent_ids:
            raise RequiredArgument(
                function=cls.delete_agents.__name__, argument="agent_ids"
            )

        cmd = XmlCommand("delete_agent")
        xml_agents = cmd.add_element("agents")

        for agent_id in agent_ids:
            xml_agents.add_element("agent", attrs={"id": agent_id})

        return cmd

    @classmethod
    def modify_agent_control_scan_config(
        cls,
        agent_control_id: EntityID,
        config_defaults: Mapping[str, Any],
    ) -> Request:
        """
        Modify agent control scan config.

        Args:
            agent_control_id: The agent control UUID.
            config_defaults: Nested config, e.g.:
                {
                    "agent_defaults": {
                        "agent_control": {
                            "retry": {
                                "attempts": 6,
                                "delay_in_seconds": 60,
                                "max_jitter_in_seconds": 10,
                            }
                        },
                        "agent_script_executor": {
                            "bulk_size": 2,
                            "bulk_throttle_time_in_ms": 300,
                            "indexer_dir_depth": 100,
                            "scheduler_cron_time": ["0 */12 * * *"],
                        },
                        "heartbeat": {
                            "interval_in_seconds": 300,
                            "miss_until_inactive": 1,
                        },
                    },
                    "agent_control_defaults": {
                        "update_to_latest": False,
                    },
                }
        """
        if not agent_control_id:
            raise RequiredArgument(
                function=cls.modify_agent_control_scan_config.__name__,
                argument="agent_control_id",
            )
        if not config_defaults:
            raise RequiredArgument(
                function=cls.modify_agent_control_scan_config.__name__,
                argument="config_defaults",
            )

        cls._validate_config_defaults(
            config_defaults,
            caller=cls.modify_agent_control_scan_config.__name__,
        )

        cmd = XmlCommand("modify_agent_control_scan_config")
        cmd.set_attribute("agent_control_id", str(agent_control_id))
        cls._append_config_defaults(cmd, config_defaults)

        return cmd

    @classmethod
    def sync_agents(cls) -> Request:
        """Trigger agents synchronization from all agent controllers."""
        return XmlCommand("sync_agents")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetNotesTestMixin:
    def test_get_notes(self):
        self.gmp.get_notes()

        self.connection.send.has_been_called_with(b"<get_notes/>")

    def test_get_notes_with_filter_string(self):
        self.gmp.get_notes(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_notes filter="foo=bar"/>'
        )

    def test_get_notes_with_filter_id(self):
        self.gmp.get_notes(filter_id="f1")

        self.connection.send.has_been_called_with(b'<get_notes filt_id="f1"/>')

    def test_get_notes_with_details(self):
        self.gmp.get_notes(details=True)

        self.connection.send.has_been_called_with(b'<get_notes details="1"/>')

        self.gmp.get_notes(details=False)

        self.connection.send.has_been_called_with(b'<get_notes details="0"/>')

    def test_get_notes_with_result(self):
        self.gmp.get_notes(result=True)

        self.connection.send.has_been_called_with(b'<get_notes result="1"/>')

        self.gmp.get_notes(result=False)

        self.connection.send.has_been_called_with(b'<get_notes result="0"/>')



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetScanConfigsTestMixin:
    def test_get_scan_configs_simple(self):
        self.gmp.get_scan_configs()

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan"/>'
        )

    def test_get_scan_configs_with_filter_string(self):
        self.gmp.get_scan_configs(filter_string="name=foo")

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" filter="name=foo"/>'
        )

    def test_get_scan_configs_with_filter_id(self):
        self.gmp.get_scan_configs(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" filt_id="f1"/>'
        )

    def test_get_scan_configs_from_trash(self):
        self.gmp.get_scan_configs(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" trash="1"/>'
        )

    def test_get_scan_configs_with_details(self):
        self.gmp.get_scan_configs(details=True)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" details="1"/>'
        )

    def test_get_scan_configs_without_details(self):
        self.gmp.get_scan_configs(details=False)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" details="0"/>'
        )

    def test_get_scan_configs_with_families(self):
        self.gmp.get_scan_configs(families=True)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" families="1"/>'
        )

    def test_get_scan_configs_without_families(self):
        self.gmp.get_scan_configs(families=False)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" families="0"/>'
        )

    def test_get_scan_configs_with_preferences(self):
        self.gmp.get_scan_configs(preferences=True)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" preferences="1"/>'
        )

    def test_get_scan_configs_without_preferences(self):
        self.gmp.get_scan_configs(preferences=False)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" preferences="0"/>'
        )

    def test_get_scan_configs_with_tasks(self):
        self.gmp.get_scan_configs(tasks=True)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" tasks="1"/>'
        )

    def test_get_scan_configs_without_tasks(self):
        self.gmp.get_scan_configs(tasks=False)

        self.connection.send.has_been_called_with(
            b'<get_configs usage_type="scan" tasks="0"/>'
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from decimal import Decimal

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import Overrides


class OverridesTestCase(unittest.TestCase):
    def test_create_override(self):
        request = Overrides.create_override("foo", nvt_oid="oid1")

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"</create_override>",
        )

    def test_create_override_missing_text(self):
        with self.assertRaises(RequiredArgument):
            Overrides.create_override(None, "od1")

        with self.assertRaises(RequiredArgument):
            Overrides.create_override("", "oid1")

    def test_create_override_missing_nvt_oid(self):
        with self.assertRaises(RequiredArgument):
            Overrides.create_override("foo", None)

        with self.assertRaises(RequiredArgument):
            Overrides.create_override("foo", "")

    def test_create_override_with_hosts(self):
        request = Overrides.create_override("foo", nvt_oid="oid1", hosts=[])

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"</create_override>",
        )

        request = Overrides.create_override(
            "foo", nvt_oid="oid1", hosts=["h1", "h2"]
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<hosts>h1,h2</hosts>"
            b"</create_override>",
        )

    def test_create_override_with_port(self):
        request = Overrides.create_override(
            "foo", nvt_oid="oid1", port="666/udp"
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<port>666/udp</port>"
            b"</create_override>",
        )

    def test_create_override_with_result_id(self):
        request = Overrides.create_override(
            "foo", nvt_oid="oid1", result_id="r1"
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b'<result id="r1"/>'
            b"</create_override>",
        )

    def test_create_override_with_task_id(self):
        request = Overrides.create_override("foo", nvt_oid="oid1", task_id="t1")

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b'<task id="t1"/>'
            b"</create_override>",
        )

    def test_create_override_with_severity(self):
        request = Overrides.create_override(
            "foo", nvt_oid="oid1", severity="5.5"
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<severity>5.5</severity>"
            b"</create_override>",
        )

        request = Overrides.create_override("foo", nvt_oid="oid1", severity=5.5)

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<severity>5.5</severity>"
            b"</create_override>",
        )

        request = Overrides.create_override(
            "foo", nvt_oid="oid1", severity=Decimal("5.5")
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<severity>5.5</severity>"
            b"</create_override>",
        )

    def test_create_override_with_new_severity(self):
        request = Overrides.create_override(
            "foo", nvt_oid="oid1", new_severity="5.5"
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<new_severity>5.5</new_severity>"
            b"</create_override>",
        )

        request = Overrides.create_override(
            "foo", nvt_oid="oid1", new_severity=5.5
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<new_severity>5.5</new_severity>"
            b"</create_override>",
        )

        request = Overrides.create_override(
            "foo", nvt_oid="oid1", new_severity=Decimal("5.5")
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<new_severity>5.5</new_severity>"
            b"</create_override>",
        )

    def test_create_override_with_days_active(self):
        request = Overrides.create_override(
            "foo", nvt_oid="oid1", days_active=0
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<active>0</active>"
            b"</create_override>",
        )

        request = Overrides.create_override(
            "foo", nvt_oid="oid1", days_active=5
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<active>5</active>"
            b"</create_override>",
        )

        request = Overrides.create_override(
            "foo", nvt_oid="oid1", days_active=-1
        )

        self.assertEqual(
            bytes(request),
            b"<create_override>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<active>-1</active>"
            b"</create_override>",
        )

    def test_create_override_with_invalid_port(self):
        with self.assertRaises(InvalidArgument):
            Overrides.create_override("foo", nvt_oid="oid1", port="invalid")

        with self.assertRaises(InvalidArgument):
            Overrides.create_override("foo", nvt_oid="oid1", port="123")

        with self.assertRaises(InvalidArgument):
            Overrides.create_override(
                text="foo", nvt_oid="oid1", port="tcp/123"
            )

    def test_modify_override(self):
        request = Overrides.modify_override("o1", "foo")

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"</modify_override>",
        )

    def test_modify_override_missing_override_id(self):
        with self.assertRaises(RequiredArgument):
            Overrides.modify_override(None, "foo")

        with self.assertRaises(RequiredArgument):
            Overrides.modify_override("", "foo")

    def test_modify_override_missing_text(self):
        with self.assertRaises(RequiredArgument):
            Overrides.modify_override("o1", None)

        with self.assertRaises(RequiredArgument):
            Overrides.modify_override("o1", "")

    def test_modify_override_with_days_active(self):
        request = Overrides.modify_override("o1", "foo", days_active=0)

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<active>0</active>"
            b"</modify_override>",
        )

        request = Overrides.modify_override("o1", "foo", days_active=5)

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<active>5</active>"
            b"</modify_override>",
        )

        request = Overrides.modify_override("o1", "foo", days_active=-1)

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<active>-1</active>"
            b"</modify_override>",
        )

    def test_modify_override_with_port(self):
        request = Overrides.modify_override("o1", "foo", port="666/udp")

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<port>666/udp</port>"
            b"</modify_override>",
        )

    def test_modify_override_with_hosts(self):
        request = Overrides.modify_override("o1", "foo", hosts=["h1"])

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<hosts>h1</hosts>"
            b"</modify_override>",
        )

        request = Overrides.modify_override("o1", "foo", hosts=["h1", "h2"])

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<hosts>h1,h2</hosts>"
            b"</modify_override>",
        )

    def test_modify_override_with_result_id(self):
        request = Overrides.modify_override("o1", "foo", result_id="r1")

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b'<result id="r1"/>'
            b"</modify_override>",
        )

    def test_modify_override_with_task_id(self):
        request = Overrides.modify_override("o1", "foo", task_id="t1")

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b'<task id="t1"/>'
            b"</modify_override>",
        )

    def test_modify_override_with_severity(self):
        request = Overrides.modify_override("o1", "foo", severity="5.5")

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_override>",
        )

        request = Overrides.modify_override("o1", "foo", severity=5.5)

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_override>",
        )

        request = Overrides.modify_override(
            "o1", "foo", severity=Decimal("5.5")
        )

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_override>",
        )

    def test_modify_override_with_new_severity(self):
        request = Overrides.modify_override("o1", "foo", new_severity="5.5")

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<new_severity>5.5</new_severity>"
            b"</modify_override>",
        )

        request = Overrides.modify_override("o1", "foo", new_severity=5.5)

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<new_severity>5.5</new_severity>"
            b"</modify_override>",
        )

        request = Overrides.modify_override(
            "o1", "foo", new_severity=Decimal("5.5")
        )

        self.assertEqual(
            bytes(request),
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<new_severity>5.5</new_severity>"
            b"</modify_override>",
        )

    def test_modify_override_with_invalid_port(self):
        with self.assertRaises(InvalidArgument):
            Overrides.modify_override("o1", "foo", port="invalid")

        with self.assertRaises(InvalidArgument):
            Overrides.modify_override("o1", "foo", port="123")

        with self.assertRaises(InvalidArgument):
            Overrides.modify_override("o1", "foo", port="tcp/123")

    def test_clone_override(self):
        request = Overrides.clone_override("o1")

        self.assertEqual(
            bytes(request),
            b"<create_override><copy>o1</copy></create_override>",
        )

    def test_clone_override_missing_override_id(self):
        with self.assertRaises(RequiredArgument):
            Overrides.clone_override(None)

        with self.assertRaises(RequiredArgument):
            Overrides.clone_override("")

    def test_delete_override(self):
        request = Overrides.delete_override("o1")

        self.assertEqual(
            bytes(request),
            b'<delete_override override_id="o1" ultimate="0"/>',
        )

    def test_delete_override_ultimate(self):
        request = Overrides.delete_override("o1", ultimate=True)

        self.assertEqual(
            bytes(request),
            b'<delete_override override_id="o1" ultimate="1"/>',
        )

        request = Overrides.delete_override("o1", ultimate=False)

        self.assertEqual(
            bytes(request),
            b'<delete_override override_id="o1" ultimate="0"/>',
        )

    def test_delete_override_missing_override_id(self):
        with self.assertRaises(RequiredArgument):
            Overrides.delete_override(None)

        with self.assertRaises(RequiredArgument):
            Overrides.delete_override("")

    def test_get_overrides(self):
        request = Overrides.get_overrides()

        self.assertEqual(
            bytes(request),
            b"<get_overrides/>",
        )

    def test_get_overrides_with_filter_string(self):
        request = Overrides.get_overrides(filter_string="foo=bar")

        self.assertEqual(bytes(request), b'<get_overrides filter="foo=bar"/>')

    def test_get_overrides_with_filter_id(self):
        request = Overrides.get_overrides(filter_id="f1")

        self.assertEqual(bytes(request), b'<get_overrides filt_id="f1"/>')

    def test_get_overrides_with_details(self):
        request = Overrides.get_overrides(details=True)

        self.assertEqual(bytes(request), b'<get_overrides details="1"/>')

        request = Overrides.get_overrides(details=False)

        self.assertEqual(bytes(request), b'<get_overrides details="0"/>')

    def test_get_overrides_with_result(self):
        request = Overrides.get_overrides(result=True)

        self.assertEqual(bytes(request), b'<get_overrides result="1"/>')

        request = Overrides.get_overrides(result=False)

        self.assertEqual(bytes(request), b'<get_overrides result="0"/>')

    def test_get_override(self):
        request = Overrides.get_override("o1")

        self.assertEqual(
            bytes(request),
            b'<get_overrides override_id="o1" details="1"/>',
        )

    def test_get_override_missing_override_id(self):
        with self.assertRaises(RequiredArgument):
            Overrides.get_override(None)

        with self.assertRaises(RequiredArgument):
            Overrides.get_override("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v224 import SystemReports


class SystemReportsTestCase(unittest.TestCase):
    def test_get_system_reports(self):
        request = SystemReports.get_system_reports()

        self.assertEqual(bytes(request), b"<get_system_reports/>")

    def test_system_reports_with_name(self):
        request = SystemReports.get_system_reports(name="foo")

        self.assertEqual(bytes(request), b'<get_system_reports name="foo"/>')

    def test_system_reports_with_slave_id(self):
        request = SystemReports.get_system_reports(slave_id="s1")

        self.assertEqual(bytes(request), b'<get_system_reports slave_id="s1"/>')

    def test_system_reports_with_brief(self):
        request = SystemReports.get_system_reports(brief=True)

        self.assertEqual(bytes(request), b'<get_system_reports brief="1"/>')

        request = SystemReports.get_system_reports(brief=False)

        self.assertEqual(bytes(request), b'<get_system_reports brief="0"/>')

    def test_system_reports_with_duration(self):
        request = SystemReports.get_system_reports(duration=3600)

        self.assertEqual(
            bytes(request), b'<get_system_reports duration="3600"/>'
        )

    def test_system_reports_with_invalid_duration(self):
        with self.assertRaises(InvalidArgument):
            SystemReports.get_system_reports(duration="")

    def test_system_reports_with_start_time(self):
        request = SystemReports.get_system_reports(start_time="01-01-2019")

        self.assertEqual(
            bytes(request), b'<get_system_reports start_time="01-01-2019"/>'
        )

    def test_system_reports_with_end_time(self):
        request = SystemReports.get_system_reports(end_time="01-01-2019")

        self.assertEqual(
            bytes(request), b'<get_system_reports end_time="01-01-2019"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_user import GmpCloneUserTestMixin
from .test_create_user import GmpCreateUserTestMixin
from .test_delete_user import GmpDeleteUserTestMixin
from .test_get_user import GmpGetUserTestMixin
from .test_get_users import GmpGetUsersTestMixin
from .test_modify_user import GmpModifyUserTestMixin

__all__ = (
    "GmpCloneUserTestMixin",
    "GmpCreateUserTestMixin",
    "GmpDeleteUserTestMixin",
    "GmpGetUserTestMixin",
    "GmpGetUsersTestMixin",
    "GmpModifyUserTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpDeletePortRangeTestMixin:
    def test_delete(self):
        self.gmp.delete_port_range("a1")

        self.connection.send.has_been_called_with(
            b'<delete_port_range port_range_id="a1"/>'
        )

    def test_missing_id(self):
        with self.assertRaises(GvmError):
            self.gmp.delete_port_range(None)

        with self.assertRaises(GvmError):
            self.gmp.delete_port_range("")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ....gmpv224.entities.scanners import (
    GmpCloneScannerTestMixin,
    GmpDeleteScannerTestMixin,
    GmpGetScannersTestMixin,
    GmpGetScannerTestMixin,
    GmpVerifyScannerTestMixin,
)
from .test_create_scanner import GmpCreateScannerTestMixin
from .test_modify_scanner import GmpModifyScannerTestMixin

__all__ = (
    "GmpCloneScannerTestMixin",
    "GmpCreateScannerTestMixin",
    "GmpDeleteScannerTestMixin",
    "GmpGetScannerTestMixin",
    "GmpGetScannersTestMixin",
    "GmpModifyScannerTestMixin",
    "GmpVerifyScannerTestMixin",
)



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpnext import GMPTestCase
from .agent_installers.test_get_agent_installer import (
    GmpGetAgentInstallerTestMixin,
)
from .agent_installers.test_get_agent_installer_file import (
    GmpGetAgentInstallerFileTestMixin,
)
from .agent_installers.test_get_agent_installers import (
    GmpGetAgentInstallersTestMixin,
)


class GMPGetAgentInstallerTestCase(GmpGetAgentInstallerTestMixin, GMPTestCase):
    pass


class GMPGetAgentInstallerFileTestCase(
    GmpGetAgentInstallerFileTestMixin, GMPTestCase
):
    pass


class GMPGetAgentInstallersTestCase(
    GmpGetAgentInstallersTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from unittest.mock import MagicMock

from gvm.connections import (
    DEFAULT_TIMEOUT,
    GvmConnection,
)
from gvm.connections._connection import AbstractGvmConnection
from gvm.errors import GvmError


class TestConnection(AbstractGvmConnection):
    def connect(self) -> None:
        pass


class GvmConnectionTestCase(unittest.TestCase):
    # pylint: disable=protected-access
    def test_init_no_args(self):
        connection = TestConnection()

        self.assertIsNone(connection._socket)
        self.assertEqual(connection._timeout, DEFAULT_TIMEOUT)

    def test_init_with_none(self):
        connection = TestConnection(timeout=None)

        self.assertIsNone(connection._socket)
        self.assertEqual(connection._timeout, DEFAULT_TIMEOUT)

    def test_read_no_data(self):
        read_mock = MagicMock()
        read_mock.return_value = None

        connection = TestConnection()
        connection._read = read_mock

        with self.assertRaises(GvmError, msg="Remote closed the connection"):
            connection.read()

    def test_read_trigger_timeout(self):
        # mocking the response into two parts, so we run into the timeout
        # check in the loop
        read_mock = MagicMock()
        read_mock.side_effect = [b"<foo>xyz<bar></bar>", b"</foo>"]

        connection = TestConnection(timeout=0)
        connection._read = read_mock

        with self.assertRaises(
            GvmError, msg="Timeout while reading the response"
        ):
            connection.read()

    def test_is_gvm_connection(self):
        connection = TestConnection()
        self.assertTrue(isinstance(connection, GvmConnection))



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_report_config import GMPCloneReportConfigTestMixin
from .test_create_report_config import GMPCreateReportConfigTestMixin
from .test_delete_report_config import GMPDeleteReportConfigTestMixin
from .test_get_report_config import GMPGetReportConfigTestMixin
from .test_get_report_configs import GMPGetReportConfigsTestMixin
from .test_modify_report_config import GMPModifyReportConfigTestMixin

__all__ = (
    "GMPCloneReportConfigTestMixin",
    "GMPCreateReportConfigTestMixin",
    "GMPDeleteReportConfigTestMixin",
    "GMPGetReportConfigTestMixin",
    "GMPGetReportConfigsTestMixin",
    "GMPModifyReportConfigTestMixin",
)



#  SPDX-FileCopyrightText: 2026 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetIntegrationConfigTestMixin:
    def test_get_integration_config(self):
        self.gmp.get_integration_config("uuid")

        self.connection.send.has_been_called_with(
            b'<get_integration_configs integration_config_id="uuid"/>'
        )
        self.gmp.get_integration_config(integration_config_id="uuid")

        self.connection.send.has_been_called_with(
            b'<get_integration_configs integration_config_id="uuid"/>'
        )

    def test_get_integration_config_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_integration_config(integration_config_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_integration_config("")

    def test_get_integration_config_with_details(self):
        self.gmp.get_integration_config(
            integration_config_id="uuid", details=True
        )

        self.connection.send.has_been_called_with(
            b'<get_integration_configs integration_config_id="uuid" details="1"/>'
        )

        self.gmp.get_integration_config(
            integration_config_id="uuid", details=False
        )

        self.connection.send.has_been_called_with(
            b'<get_integration_configs integration_config_id="uuid" details="0"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .roles import (
    GmpCloneRoleTestMixin,
    GmpCreateRoleTestMixin,
    GmpDeleteRoleTestMixin,
    GmpGetRolesTestMixin,
    GmpGetRoleTestMixin,
    GmpModifyRoleTestMixin,
)


class Gmpv224DeleteRoleTestCase(GmpDeleteRoleTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetRoleTestCase(GmpGetRoleTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetRolesTestCase(GmpGetRolesTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CloneRoleTestCase(GmpCloneRoleTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateRoleTestCase(GmpCreateRoleTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyRoleTestCase(GmpModifyRoleTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v226 import CredentialFormat


class GetCredentialFormatFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            CredentialFormat.from_string("foo")

    def test_none_or_empty(self):
        ct = CredentialFormat.from_string(None)
        self.assertIsNone(ct)
        ct = CredentialFormat.from_string("")
        self.assertIsNone(ct)

    def test_key(self):
        ct = CredentialFormat.from_string("key")
        self.assertEqual(ct, CredentialFormat.KEY)

    def test_rpm(self):
        ct = CredentialFormat.from_string("rpm")
        self.assertEqual(ct, CredentialFormat.RPM)

    def test_deb(self):
        ct = CredentialFormat.from_string("deb")
        self.assertEqual(ct, CredentialFormat.DEB)

    def test_exe(self):
        ct = CredentialFormat.from_string("exe")
        self.assertEqual(ct, CredentialFormat.EXE)

    def test_pem(self):
        ct = CredentialFormat.from_string("pem")
        self.assertEqual(ct, CredentialFormat.PEM)



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyCredentialStoreCredentialTestMixin:
    def test_modify_cs_credential(self):
        self.gmp.modify_credential_store_credential(credential_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1"/>'
        )

    def test_modify_cs_credential_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential_store_credential(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential_store_credential("")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential_store_credential(credential_id="")

    def test_modify_cs_credential_with_name(self):
        self.gmp.modify_credential_store_credential(
            credential_id="c1", name="foo"
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<name>foo</name>"
            b"</modify_credential>"
        )

    def test_modify_cs_credential_with_comment(self):
        self.gmp.modify_credential_store_credential(
            credential_id="c1", comment="foo"
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<comment>foo</comment>"
            b"</modify_credential>"
        )

    def test_modify_cs_credential_with_credential_store_id(self):
        self.gmp.modify_credential_store_credential(
            credential_id="c1", credential_store_id="foo"
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<credential_store_id>foo</credential_store_id>"
            b"</modify_credential>"
        )

    def test_modify_cs_credential_with_vault_id(self):
        self.gmp.modify_credential_store_credential(
            credential_id="c1", vault_id="foo"
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<vault_id>foo</vault_id>"
            b"</modify_credential>"
        )

    def test_modify_cs_credential_with_host_identifier(self):
        self.gmp.modify_credential_store_credential(
            credential_id="c1", host_identifier="foo"
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<host_identifier>foo</host_identifier>"
            b"</modify_credential>"
        )

    def test_modify_cs_credential_with_all(self):
        self.gmp.modify_credential_store_credential(
            credential_id="c1",
            name="foo_name",
            comment="foo_comment",
            credential_store_id="foo_csid",
            vault_id="foo_vid",
            host_identifier="foo_hid",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<name>foo_name</name>"
            b"<comment>foo_comment</comment>"
            b"<credential_store_id>foo_csid</credential_store_id>"
            b"<vault_id>foo_vid</vault_id>"
            b"<host_identifier>foo_hid</host_identifier>"
            b"</modify_credential>"
        )
